@@ -1,8 +1,11 @@
-__version__ = "0.14.7"
+__version__ = "1.0.0"
 
+from bigtree.binarytree.binarytree import BinaryTree
 from bigtree.binarytree.construct import list_to_binarytree
 from bigtree.dag.construct import dataframe_to_dag, dict_to_dag, list_to_dag
+from bigtree.dag.dag import DAG
 from bigtree.dag.export import dag_to_dataframe, dag_to_dict, dag_to_dot, dag_to_list
+from bigtree.dag.parsing import get_path_dag
 from bigtree.node.basenode import BaseNode
 from bigtree.node.binarynode import BinaryNode
 from bigtree.node.dagnode import DAGNode
@@ -13,34 +16,59 @@ from bigtree.tree.construct import (
     add_dict_to_tree_by_name,
     add_dict_to_tree_by_path,
     add_path_to_tree,
+    add_polars_to_tree_by_name,
+    add_polars_to_tree_by_path,
     dataframe_to_tree,
     dataframe_to_tree_by_relation,
     dict_to_tree,
     list_to_tree,
     list_to_tree_by_relation,
+    nested_dict_key_to_tree,
     nested_dict_to_tree,
+    newick_to_tree,
+    polars_to_tree,
+    polars_to_tree_by_relation,
+    render_tree,
     str_to_tree,
 )
 from bigtree.tree.export import (
+    hprint_tree,
+    hyield_tree,
     print_tree,
     tree_to_dataframe,
     tree_to_dict,
     tree_to_dot,
     tree_to_mermaid,
     tree_to_nested_dict,
+    tree_to_nested_dict_key,
+    tree_to_newick,
     tree_to_pillow,
+    tree_to_pillow_graph,
+    tree_to_polars,
+    tree_to_vis,
+    vprint_tree,
+    vyield_tree,
     yield_tree,
 )
-from bigtree.tree.helper import clone_tree, get_tree_diff, prune_tree
+from bigtree.tree.helper import (
+    clone_tree,
+    get_subtree,
+    get_tree_diff,
+    get_tree_diff_dataframe,
+    prune_tree,
+)
 from bigtree.tree.modify import (
     copy_and_replace_nodes_from_tree_to_tree,
     copy_nodes,
     copy_nodes_from_tree_to_tree,
     copy_or_shift_logic,
+    merge_trees,
     replace_logic,
     shift_and_replace_nodes,
     shift_nodes,
 )
+from bigtree.tree.parsing import get_common_ancestors, get_path
+from bigtree.tree.query import query_tree
 from bigtree.tree.search import (
     find,
     find_attr,
@@ -54,15 +82,39 @@ from bigtree.tree.search import (
     find_path,
     find_paths,
     find_relative_path,
+    find_relative_paths,
     findall,
 )
-from bigtree.utils.exceptions import (
-    CorruptedTreeError,
-    DuplicatedNodeError,
-    LoopError,
-    NotFoundError,
-    SearchError,
-    TreeError,
+from bigtree.tree.tree import Tree
+from bigtree.utils.constants import (
+    ANSIBorderStyle,
+    ANSIHPrintStyle,
+    ANSIPrintStyle,
+    ANSIVPrintStyle,
+    ASCIIBorderStyle,
+    ASCIIHPrintStyle,
+    ASCIIPrintStyle,
+    ASCIIVPrintStyle,
+    BaseHPrintStyle,
+    BasePrintStyle,
+    BaseVPrintStyle,
+    BorderStyle,
+    ConstBoldBorderStyle,
+    ConstBoldHPrintStyle,
+    ConstBoldPrintStyle,
+    ConstBoldVPrintStyle,
+    ConstBorderStyle,
+    ConstHPrintStyle,
+    ConstPrintStyle,
+    ConstVPrintStyle,
+    DoubleBorderStyle,
+    DoubleHPrintStyle,
+    DoublePrintStyle,
+    DoubleVPrintStyle,
+    RoundedBorderStyle,
+    RoundedHPrintStyle,
+    RoundedPrintStyle,
+    RoundedVPrintStyle,
 )
 from bigtree.utils.groot import speak_like_groot, whoami
 from bigtree.utils.iterators import (
@@ -75,6 +127,6 @@ from bigtree.utils.iterators import (
     zigzag_iter,
     zigzaggroup_iter,
 )
-from bigtree.utils.plot import reingold_tilford
+from bigtree.utils.plot import plot_tree, reingold_tilford
 from bigtree.workflows.app_calendar import Calendar
 from bigtree.workflows.app_todo import AppToDo
