@@ -1,70 +1,77 @@
 from __future__ import annotations
 
 import copy
-from typing import Any, Dict, Generator, Iterable, List, Optional, Set, Tuple, TypeVar
+import heapq
+from typing import Any, Generator, Iterable, Mapping, TypeVar
 
-from bigtree.utils.exceptions import CorruptedTreeError, LoopError, TreeError
-from bigtree.utils.iterators import preorder_iter
+from bigtree.globals import ASSERTIONS
+from bigtree.utils import exceptions, iterators
+
+try:
+    import matplotlib.pyplot as plt
+except ImportError:  # pragma: no cover
+    plt = None
 
 
 class BaseNode:
     """
-    BaseNode extends any Python class to a tree node.
-    Nodes can have attributes if they are initialized from `Node`, *dictionary*, or *pandas DataFrame*.
+    BaseNode extends any Python class to a tree node. Nodes can have attributes if they are initialized from `Node`,
+    *dictionary*, or *pandas DataFrame*.
 
-    Nodes can be linked to each other with `parent` and `children` setter methods,
-    or using bitshift operator with the convention `parent_node >> child_node` or `child_node << parent_node`.
+    Nodes can be linked to each other with `parent` and `children` setter methods, or using bitshift operator with the
+    convention `parent_node >> child_node` or `child_node << parent_node`.
 
-    >>> from bigtree import Node, print_tree
-    >>> root = Node("a", age=90)
-    >>> b = Node("b", age=65)
-    >>> c = Node("c", age=60)
-    >>> d = Node("d", age=40)
-    >>> root.children = [b, c]
-    >>> d.parent = b
-    >>> print_tree(root, attr_list=["age"])
-    a [age=90]
-    ├── b [age=65]
-    │   └── d [age=40]
-    └── c [age=60]
+    Examples:
+        >>> from bigtree import Node, print_tree
+        >>> root = Node("a", age=90)
+        >>> b = Node("b", age=65)
+        >>> c = Node("c", age=60)
+        >>> d = Node("d", age=40)
+        >>> root.children = [b, c]
+        >>> d.parent = b
+        >>> print_tree(root, attr_list=["age"])
+        a [age=90]
+        ├── b [age=65]
+        │   └── d [age=40]
+        └── c [age=60]
 
-    >>> from bigtree import Node
-    >>> root = Node("a", age=90)
-    >>> b = Node("b", age=65)
-    >>> c = Node("c", age=60)
-    >>> d = Node("d", age=40)
-    >>> root >> b
-    >>> root >> c
-    >>> d << b
-    >>> print_tree(root, attr_list=["age"])
-    a [age=90]
-    ├── b [age=65]
-    │   └── d [age=40]
-    └── c [age=60]
+        >>> from bigtree import Node
+        >>> root = Node("a", age=90)
+        >>> b = Node("b", age=65)
+        >>> c = Node("c", age=60)
+        >>> d = Node("d", age=40)
+        >>> root >> b
+        >>> root >> c
+        >>> d << b
+        >>> print_tree(root, attr_list=["age"])
+        a [age=90]
+        ├── b [age=65]
+        │   └── d [age=40]
+        └── c [age=60]
 
-    Directly passing `parent` argument.
+        Directly passing `parent` argument.
 
-    >>> from bigtree import Node
-    >>> root = Node("a")
-    >>> b = Node("b", parent=root)
-    >>> c = Node("c", parent=root)
-    >>> d = Node("d", parent=b)
+        >>> from bigtree import Node
+        >>> root = Node("a")
+        >>> b = Node("b", parent=root)
+        >>> c = Node("c", parent=root)
+        >>> d = Node("d", parent=b)
 
-    Directly passing `children` argument.
+        Directly passing `children` argument.
 
-    >>> from bigtree import Node
-    >>> d = Node("d")
-    >>> c = Node("c")
-    >>> b = Node("b", children=[d])
-    >>> a = Node("a", children=[b, c])
+        >>> from bigtree import Node
+        >>> d = Node("d")
+        >>> c = Node("c")
+        >>> b = Node("b", children=[d])
+        >>> a = Node("a", children=[b, c])
 
     **BaseNode Creation**
 
-    Node can be created by instantiating a `BaseNode` class or by using a *dictionary*.
-    If node is created with dictionary, all keys of dictionary will be stored as class attributes.
+    Node can be created by instantiating a `BaseNode` class or by using a *dictionary*. If node is created with
+    dictionary, all keys of dictionary will be stored as class attributes.
 
-    >>> from bigtree import Node
-    >>> root = Node.from_dict({"name": "a", "age": 90})
+        >>> from bigtree import Node
+        >>> root = Node.from_dict({"name": "a", "age": 90})
 
     **BaseNode Attributes**
 
@@ -90,8 +97,9 @@ class BaseNode:
     2. ``is_root``: Get indicator if self is root node
     3. ``is_leaf``: Get indicator if self is leaf node
     4. ``root``: Get root node of tree
-    5. ``depth``: Get depth of self
-    6. ``max_depth``: Get maximum depth from root to leaf node
+    5. ``diameter``: Get diameter of self
+    6. ``depth``: Get depth of self
+    7. ``max_depth``: Get maximum depth from root to leaf node
 
     **BaseNode Methods**
 
@@ -108,9 +116,11 @@ class BaseNode:
     3. ``set_attrs(attrs: dict)``: Set node attribute name(s) and value(s)
     4. ``go_to(node: Self)``: Get a path from own node to another node from same tree
     5. ``append(node: Self)``: Add child to node
-    6. ``extend(nodes: List[Self])``: Add multiple children to node
+    6. ``extend(nodes: list[Self])``: Add multiple children to node
     7. ``copy()``: Deep copy self
     8. ``sort()``: Sort child nodes
+    9. ``plot()``: Plot tree in line form
+    10. ``query(query: str)``: Filter tree using Tree Query Language
 
     ----
 
@@ -118,12 +128,12 @@ class BaseNode:
 
     def __init__(
         self,
-        parent: Optional[T] = None,
-        children: Optional[List[T]] = None,
+        parent: T | None = None,
+        children: list[T] | None = None,
         **kwargs: Any,
     ):
-        self.__parent: Optional[T] = None
-        self.__children: List[T] = []
+        self.__parent: T | None = None
+        self.__children: list[T] = []
         if children is None:
             children = []
         self.parent = parent
@@ -136,10 +146,10 @@ class BaseNode:
 
     @staticmethod
     def __check_parent_type(new_parent: T) -> None:
-        """Check parent type
+        """Check parent type.
 
         Args:
-            new_parent (Self): parent node
+            new_parent: parent node
         """
         if not (isinstance(new_parent, BaseNode) or new_parent is None):
             raise TypeError(
@@ -147,41 +157,44 @@ class BaseNode:
             )
 
     def __check_parent_loop(self, new_parent: T) -> None:
-        """Check parent type
+        """Check parent type.
 
         Args:
-            new_parent (Self): parent node
+            new_parent: parent node
         """
         if new_parent is not None:
             if new_parent is self:
-                raise LoopError("Error setting parent: Node cannot be parent of itself")
+                raise exceptions.LoopError(
+                    "Error setting parent: Node cannot be parent of itself"
+                )
             if any(
                 ancestor is self
                 for ancestor in new_parent.ancestors
                 if new_parent.ancestors
             ):
-                raise LoopError(
+                raise exceptions.LoopError(
                     "Error setting parent: Node cannot be ancestor of itself"
                 )
 
     @property
-    def parent(self: T) -> Optional[T]:
-        """Get parent node
+    def parent(self: T) -> T | None:
+        """Get parent node.
 
         Returns:
-            (Optional[Self])
+            Parent node, none if the node is root
         """
         return self.__parent
 
     @parent.setter
     def parent(self: T, new_parent: T) -> None:
-        """Set parent node
+        """Set parent node.
 
         Args:
-            new_parent (Self): parent node
+            new_parent: parent node
         """
-        self.__check_parent_type(new_parent)
-        self.__check_parent_loop(new_parent)
+        if ASSERTIONS:
+            self.__check_parent_type(new_parent)
+            self.__check_parent_loop(new_parent)
 
         current_parent = self.parent
         current_child_idx = None
@@ -194,7 +207,7 @@ class BaseNode:
                 if not any(
                     child is self for child in current_parent.children
                 ):  # pragma: no cover
-                    raise CorruptedTreeError(
+                    raise exceptions.CorruptedTreeError(
                         "Error setting parent: Node does not exist as children of its parent"
                     )
                 current_child_idx = current_parent.__children.index(self)
@@ -216,51 +229,54 @@ class BaseNode:
             self.__parent = current_parent
             if current_child_idx is not None:
                 current_parent.__children.insert(current_child_idx, self)
-            raise TreeError(exc_info)
+            raise exceptions.TreeError(exc_info) from None
 
     def __pre_assign_parent(self, new_parent: T) -> None:
-        """Custom method to check before attaching parent
-        Can be overridden with `_BaseNode__pre_assign_parent()`
+        """Custom method to check before attaching parent. Can be overridden with `_BaseNode__pre_assign_parent()`.
 
         Args:
-            new_parent (Self): new parent to be added
+            new_parent: new parent to be added
         """
         pass
 
     def __post_assign_parent(self, new_parent: T) -> None:
-        """Custom method to check after attaching parent
-        Can be overridden with `_BaseNode__post_assign_parent()`
+        """Custom method to check after attaching parent. Can be overridden with `_BaseNode__post_assign_parent()`.
 
         Args:
-            new_parent (Self): new parent to be added
+            new_parent: new parent to be added
         """
         pass
 
     @property
     def parents(self) -> None:
-        """Do not allow `parents` attribute to be accessed"""
+        """Do not allow `parents` attribute to be accessed.
+
+        Raises:
+            AttributeError: No such attribute
+        """
         raise AttributeError(
             "Attempting to access `parents` attribute, do you mean `parent`?"
         )
 
     @parents.setter
     def parents(self, new_parent: T) -> None:
-        """Do not allow `parents` attribute to be set
+        """Do not allow `parents` attribute to be set.
 
         Args:
-            new_parent (Self): parent node
+            new_parent: parent node
+
+        Raises:
+            AttributeError: No such attribute
         """
         raise AttributeError(
             "Attempting to set `parents` attribute, do you mean `parent`?"
         )
 
-    def __check_children_type(
-        self: T, new_children: List[T] | Tuple[T] | Set[T]
-    ) -> None:
-        """Check child type
+    def __check_children_type(self: T, new_children: Iterable[T]) -> None:
+        """Check child type.
 
         Args:
-            new_children (Iterable[Self]): child node
+            new_children: child node
         """
         if (
             not isinstance(new_children, list)
@@ -272,10 +288,10 @@ class BaseNode:
             )
 
     def __check_children_loop(self: T, new_children: Iterable[T]) -> None:
-        """Check child loop
+        """Check child loop.
 
         Args:
-            new_children (Iterable[Self]): child node
+            new_children: child node
         """
         seen_children = []
         for new_child in new_children:
@@ -287,38 +303,41 @@ class BaseNode:
 
             # Check for loop and tree structure
             if new_child is self:
-                raise LoopError("Error setting child: Node cannot be child of itself")
+                raise exceptions.LoopError(
+                    "Error setting child: Node cannot be child of itself"
+                )
             if any(child is new_child for child in self.ancestors):
-                raise LoopError(
+                raise exceptions.LoopError(
                     "Error setting child: Node cannot be ancestor of itself"
                 )
 
             # Check for duplicate children
             if id(new_child) in seen_children:
-                raise TreeError(
+                raise exceptions.TreeError(
                     "Error setting child: Node cannot be added multiple times as a child"
                 )
             else:
                 seen_children.append(id(new_child))
 
     @property
-    def children(self: T) -> Tuple[T, ...]:
-        """Get child nodes
+    def children(self: T) -> tuple[T, ...]:
+        """Get child nodes.
 
         Returns:
-            (Tuple[Self, ...])
+            Child node(s)
         """
         return tuple(self.__children)
 
     @children.setter
-    def children(self: T, new_children: List[T] | Tuple[T] | Set[T]) -> None:
-        """Set child nodes
+    def children(self: T, new_children: list[T] | tuple[T] | set[T]) -> None:
+        """Set child nodes.
 
         Args:
-            new_children (List[Self]): child node
+            new_children: child node
         """
-        self.__check_children_type(new_children)
-        self.__check_children_loop(new_children)
+        if ASSERTIONS:
+            self.__check_children_type(new_children)
+            self.__check_children_loop(new_children)
         new_children = list(new_children)
 
         current_new_children = {
@@ -357,39 +376,37 @@ class BaseNode:
             self.__children = current_children
             for child in current_children:
                 child.__parent = self
-            raise TreeError(exc_info)
+            raise exceptions.TreeError(exc_info) from None
 
     @children.deleter
     def children(self) -> None:
-        """Delete child node(s)"""
+        """Delete child node(s)."""
         for child in self.children:
             child.parent.__children.remove(child)  # type: ignore
             child.__parent = None
 
     def __pre_assign_children(self: T, new_children: Iterable[T]) -> None:
-        """Custom method to check before attaching children
-        Can be overridden with `_BaseNode__pre_assign_children()`
+        """Custom method to check before attaching children. Can be overridden with `_BaseNode__pre_assign_children()`.
 
         Args:
-            new_children (Iterable[Self]): new children to be added
+            new_children: new children to be added
         """
         pass
 
     def __post_assign_children(self: T, new_children: Iterable[T]) -> None:
-        """Custom method to check after attaching children
-        Can be overridden with `_BaseNode__post_assign_children()`
+        """Custom method to check after attaching children. Can be overridden with `_BaseNode__post_assign_children()`.
 
         Args:
-            new_children (Iterable[Self]): new children to be added
+            new_children: new children to be added
         """
         pass
 
     @property
     def ancestors(self: T) -> Iterable[T]:
-        """Get iterator to yield all ancestors of self, does not include self
+        """Get iterator to yield all ancestors of self, does not include self.
 
         Returns:
-            (Iterable[Self])
+            Ancestor(s) of node excluding itself
         """
         node = self.parent
         while node is not None:
@@ -398,39 +415,43 @@ class BaseNode:
 
     @property
     def descendants(self: T) -> Iterable[T]:
-        """Get iterator to yield all descendants of self, does not include self
+        """Get iterator to yield all descendants of self, does not include self.
 
         Returns:
-            (Iterable[Self])
+            Descendant(s) of node excluding itself
         """
-        yield from preorder_iter(self, filter_condition=lambda _node: _node != self)
+        yield from iterators.preorder_iter(
+            self, filter_condition=lambda _node: _node != self
+        )
 
     @property
     def leaves(self: T) -> Iterable[T]:
-        """Get iterator to yield all leaf nodes from self
+        """Get iterator to yield all leaf nodes from self.
 
         Returns:
-            (Iterable[Self])
+            Leaf node(s) of node
         """
-        yield from preorder_iter(self, filter_condition=lambda _node: _node.is_leaf)
+        yield from iterators.preorder_iter(
+            self, filter_condition=lambda _node: _node.is_leaf
+        )
 
     @property
     def siblings(self: T) -> Iterable[T]:
-        """Get siblings of self
+        """Get siblings of self.
 
         Returns:
-            (Iterable[Self])
+            Sibling(s) of node
         """
         if self.parent is None:
             return ()
         return tuple(child for child in self.parent.children if child is not self)
 
     @property
-    def left_sibling(self: T) -> T:
-        """Get sibling left of self
+    def left_sibling(self: T) -> T | None:
+        """Get sibling left of self.
 
         Returns:
-            (Self)
+            Left sibling of node
         """
         if self.parent:
             children = self.parent.children
@@ -439,11 +460,11 @@ class BaseNode:
                 return self.parent.children[child_idx - 1]
 
     @property
-    def right_sibling(self: T) -> T:
-        """Get sibling right of self
+    def right_sibling(self: T) -> T | None:
+        """Get sibling right of self.
 
         Returns:
-            (Self)
+            Right sibling of node
         """
         if self.parent:
             children = self.parent.children
@@ -453,50 +474,81 @@ class BaseNode:
 
     @property
     def node_path(self: T) -> Iterable[T]:
-        """Get tuple of nodes starting from root
+        """Get tuple of nodes starting from root.
 
         Returns:
-            (Iterable[Self])
+            Node path from root to itself
         """
         if self.parent is None:
-            return [self]
+            return tuple([self])
         return tuple(list(self.parent.node_path) + [self])
 
     @property
     def is_root(self) -> bool:
-        """Get indicator if self is root node
+        """Get indicator if self is root node.
 
         Returns:
-            (bool)
+            Indicator if node is root node
         """
         return self.parent is None
 
     @property
     def is_leaf(self) -> bool:
-        """Get indicator if self is leaf node
+        """Get indicator if self is leaf node.
 
         Returns:
-            (bool)
+            Indicator if node is leaf node
         """
         return not len(list(self.children))
 
     @property
     def root(self: T) -> T:
-        """Get root node of tree
+        """Get root node of tree.
 
         Returns:
-            (Self)
+            Root node
         """
         if self.parent is None:
             return self
         return self.parent.root
 
     @property
-    def depth(self) -> int:
-        """Get depth of self, indexing starts from 1
+    def diameter(self) -> int:
+        """Get diameter of tree or subtree, the length of longest path between any two nodes.
 
         Returns:
-            (int)
+            Diameter of node
+        """
+        diameter = 0
+
+        if self.is_leaf:
+            return diameter
+
+        def _recursive_diameter(node: T) -> int:
+            """Recursively iterate through node and its children to get diameter of node.
+
+            Args:
+                node: current node
+
+            Returns:
+                Diameter of node
+            """
+            nonlocal diameter
+            if node.is_leaf:
+                return 1
+            child_length = [_recursive_diameter(child) for child in node.children]
+            diameter = max(diameter, sum(heapq.nlargest(2, child_length)))
+            return 1 + max(child_length)
+
+        _recursive_diameter(self)
+        return diameter
+
+    @property
+    def depth(self) -> int:
+        """Get depth of self, indexing starts from 1.
+
+        Returns:
+            Depth of node
         """
         if self.parent is None:
             return 1
@@ -504,51 +556,53 @@ class BaseNode:
 
     @property
     def max_depth(self) -> int:
-        """Get maximum depth from root to leaf node
+        """Get maximum depth from root to leaf node.
 
         Returns:
-            (int)
+            Maximum depth of tree
         """
         return max(
             [self.root.depth] + [node.depth for node in list(self.root.descendants)]
         )
 
     @classmethod
-    def from_dict(cls, input_dict: Dict[str, Any]) -> BaseNode:
-        """Construct node from dictionary, all keys of dictionary will be stored as class attributes
-        Input dictionary must have key `name` if not `Node` will not have any name
+    def from_dict(cls, input_dict: Mapping[str, Any]) -> BaseNode:
+        """Construct node from dictionary, all keys of dictionary will be stored as class attributes.
+        Input dictionary must have key `name` if not `Node` will not have any name.
 
-        >>> from bigtree import Node
-        >>> a = Node.from_dict({"name": "a", "age": 90})
+        Examples:
+            >>> from bigtree import Node
+            >>> a = Node.from_dict({"name": "a", "age": 90})
 
         Args:
-            input_dict (Dict[str, Any]): dictionary with node information, key: attribute name, value: attribute value
+            input_dict: node information, key: attribute name, value: attribute value
 
         Returns:
-            (BaseNode)
+            Base node
         """
         return cls(**input_dict)
 
     def describe(
-        self, exclude_attributes: List[str] = [], exclude_prefix: str = ""
-    ) -> List[Tuple[str, Any]]:
-        """Get node information sorted by attribute name, returns list of tuples
+        self, exclude_attributes: Iterable[str] = (), exclude_prefix: str = ""
+    ) -> list[tuple[str, Any]]:
+        """Get node information sorted by attribute name, returns list of tuples.
 
-        >>> from bigtree.node.node import Node
-        >>> a = Node('a', age=90)
-        >>> a.describe()
-        [('_BaseNode__children', []), ('_BaseNode__parent', None), ('_sep', '/'), ('age', 90), ('name', 'a')]
-        >>> a.describe(exclude_prefix="_")
-        [('age', 90), ('name', 'a')]
-        >>> a.describe(exclude_prefix="_", exclude_attributes=["name"])
-        [('age', 90)]
+        Examples:
+            >>> from bigtree.node.node import Node
+            >>> a = Node('a', age=90)
+            >>> a.describe()
+            [('_BaseNode__children', []), ('_BaseNode__parent', None), ('_sep', '/'), ('age', 90), ('name', 'a')]
+            >>> a.describe(exclude_prefix="_")
+            [('age', 90), ('name', 'a')]
+            >>> a.describe(exclude_prefix="_", exclude_attributes=["name"])
+            [('age', 90)]
 
         Args:
-            exclude_attributes (List[str]): list of attributes to exclude
-            exclude_prefix (str): prefix of attributes to exclude
+            exclude_attributes: attributes to exclude
+            exclude_prefix: prefix of attributes to exclude
 
         Returns:
-            (List[Tuple[str, Any]])
+            List of attribute name and attribute value pairs
         """
         return [
             item
@@ -558,164 +612,169 @@ class BaseNode:
         ]
 
     def get_attr(self, attr_name: str, default_value: Any = None) -> Any:
-        """Get value of node attribute
-        Returns default value if attribute name does not exist
+        """Get value of node attribute. Returns default value if attribute name does not exist.
 
-        >>> from bigtree.node.node import Node
-        >>> a = Node('a', age=90)
-        >>> a.get_attr("age")
-        90
+        Examples:
+            >>> from bigtree.node.node import Node
+            >>> a = Node('a', age=90)
+            >>> a.get_attr("age")
+            90
 
         Args:
-            attr_name (str): attribute name
-            default_value (Any): default value if attribute does not exist, defaults to None
+            attr_name: attribute name
+            default_value: default value if attribute does not exist
 
         Returns:
-            (Any)
+            Attribute value of node
         """
         try:
             return getattr(self, attr_name)
         except AttributeError:
             return default_value
 
-    def set_attrs(self, attrs: Dict[str, Any]) -> None:
-        """Set node attributes
+    def set_attrs(self, attrs: Mapping[str, Any]) -> None:
+        """Set node attributes.
 
-        >>> from bigtree.node.node import Node
-        >>> a = Node('a')
-        >>> a.set_attrs({"age": 90})
-        >>> a
-        Node(/a, age=90)
+        Examples:
+            >>> from bigtree.node.node import Node
+            >>> a = Node('a')
+            >>> a.set_attrs({"age": 90})
+            >>> a
+            Node(/a, age=90)
 
         Args:
-            attrs (Dict[str, Any]): attribute dictionary,
-                key: attribute name, value: attribute value
+            attrs: attribute information, key: attribute name, value: attribute value
         """
         self.__dict__.update(attrs)
 
     def go_to(self: T, node: T) -> Iterable[T]:
-        """Get path from current node to specified node from same tree
-
-        >>> from bigtree import Node, print_tree
-        >>> a = Node(name="a")
-        >>> b = Node(name="b", parent=a)
-        >>> c = Node(name="c", parent=a)
-        >>> d = Node(name="d", parent=b)
-        >>> e = Node(name="e", parent=b)
-        >>> f = Node(name="f", parent=c)
-        >>> g = Node(name="g", parent=e)
-        >>> h = Node(name="h", parent=e)
-        >>> print_tree(a)
-        a
-        ├── b
-        │   ├── d
-        │   └── e
-        │       ├── g
-        │       └── h
-        └── c
-            └── f
-        >>> d.go_to(d)
-        [Node(/a/b/d, )]
-        >>> d.go_to(g)
-        [Node(/a/b/d, ), Node(/a/b, ), Node(/a/b/e, ), Node(/a/b/e/g, )]
-        >>> d.go_to(f)
-        [Node(/a/b/d, ), Node(/a/b, ), Node(/a, ), Node(/a/c, ), Node(/a/c/f, )]
+        """Get path from current node to specified node from same tree, uses `get_path` function.
 
         Args:
-            node (Self): node to travel to from current node, inclusive of start and end node
+            node: node to travel to from current node, inclusive of start and end node
 
         Returns:
-            (Iterable[Self])
+            Path from current node to destination node
         """
-        if not isinstance(node, BaseNode):
-            raise TypeError(
-                f"Expect node to be BaseNode type, received input type {type(node)}"
-            )
-        if self.root != node.root:
-            raise TreeError(
-                f"Nodes are not from the same tree. Check {self} and {node}"
-            )
-        if self == node:
-            return [self]
-        self_path = [self] + list(self.ancestors)
-        node_path = ([node] + list(node.ancestors))[::-1]
-        common_nodes = set(self_path).intersection(set(node_path))
-        self_min_index, min_common_node = sorted(
-            [(self_path.index(_node), _node) for _node in common_nodes]
-        )[0]
-        node_min_index = node_path.index(min_common_node)
-        return self_path[:self_min_index] + node_path[node_min_index:]
+        from bigtree.tree.parsing import get_path
 
-    def append(self: T, other: T) -> None:
-        """Add other as child of self
+        return get_path(self, node)
+
+    def append(self: T, other: T) -> T:
+        """Add other as child of self. Can be chained.
 
         Args:
-            other (Self): other node, child to be added
+            other: other node, child to be added
         """
         other.parent = self
+        return self
 
-    def extend(self: T, others: List[T]) -> None:
-        """Add others as children of self
+    def extend(self: T, others: list[T]) -> T:
+        """Add others as children of self. Can be chained.
 
         Args:
-            others (Self): other nodes, children to be added
+            others: other nodes, children to be added
         """
         for child in others:
             child.parent = self
+        return self
 
     def copy(self: T) -> T:
-        """Deep copy self; clone self
+        """Deep copy self; clone BaseNode.
 
-        >>> from bigtree.node.node import Node
-        >>> a = Node('a')
-        >>> a_copy = a.copy()
+        Examples:
+            >>> from bigtree.node.node import Node
+            >>> a = Node('a')
+            >>> a_copy = a.copy()
 
         Returns:
-            (Self)
+            Cloned copy of node
         """
         return copy.deepcopy(self)
 
-    def sort(self: T, **kwargs: Any) -> None:
-        """Sort children, possible keyword arguments include ``key=lambda node: node.name``, ``reverse=True``
+    def sort(self: T, **kwargs: Any) -> T:
+        """Sort children, possible keyword arguments include ``key=lambda node: node.node_name``, ``reverse=True``.
+        Accepts kwargs for sort() function. Can be chained.
 
-        >>> from bigtree import Node, print_tree
-        >>> a = Node('a')
-        >>> c = Node("c", parent=a)
-        >>> b = Node("b", parent=a)
-        >>> print_tree(a)
-        a
-        ├── c
-        └── b
-        >>> a.sort(key=lambda node: node.name)
-        >>> print_tree(a)
-        a
-        ├── b
-        └── c
+        Examples:
+            >>> from bigtree import Node, print_tree
+            >>> a = Node('a')
+            >>> c = Node("c", parent=a)
+            >>> b = Node("b", parent=a)
+            >>> print_tree(a)
+            a
+            ├── c
+            └── b
+            >>> a.sort(key=lambda node: node.node_name)
+            Node(/a, )
+            >>> print_tree(a)
+            a
+            ├── b
+            └── c
         """
         children = list(self.children)
         children.sort(**kwargs)
         self.__children = children
+        return self
 
-    def __copy__(self: T) -> T:
-        """Shallow copy self
+    def plot(self, *args: Any, **kwargs: Any) -> plt.Figure:
+        """Plot tree in line form. Accepts args and kwargs for matplotlib.pyplot.plot() function.
 
-        >>> import copy
-        >>> from bigtree.node.node import Node
-        >>> a = Node('a')
-        >>> a_copy = copy.deepcopy(a)
+        Examples:
+            >>> from bigtree import list_to_tree
+            >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
+            >>> root = list_to_tree(path_list)
+            >>> root.plot("-ok")
+            <Figure size 1280x960 with 1 Axes>
+        """
+        from bigtree.utils.plot import plot_tree, reingold_tilford
+
+        if self.get_attr("x") is None or self.get_attr("y") is None:
+            reingold_tilford(self)
+        return plot_tree(self, *args, **kwargs)
+
+    def query(self, query: str, debug: bool = False) -> list[T]:
+        """Query tree using Tree Definition Language.
+
+        Examples:
+            >>> from bigtree import list_to_tree
+            >>> path_list = ["a/b/d", "a/b/e/g", "a/b/e/h", "a/c/f"]
+            >>> root = list_to_tree(path_list)
+            >>> root.query("depth == 2")
+            [Node(/a/b, ), Node(/a/c, )]
+
+        Args:
+            query: query
+            debug: if True, will print out the parsed query
 
         Returns:
-            (Self)
+            List of nodes that fulfil the condition of query
+        """
+        from bigtree.tree.query import query_tree
+
+        return query_tree(self, query, debug)
+
+    def __copy__(self: T) -> T:
+        """Shallow copy self.
+
+        Examples:
+            >>> import copy
+            >>> from bigtree.node.node import Node
+            >>> a = Node('a')
+            >>> a_copy = copy.copy(a)
+
+        Returns:
+            Shallow copy of node
         """
         obj: T = type(self).__new__(self.__class__)
         obj.__dict__.update(self.__dict__)
         return obj
 
     def __repr__(self) -> str:
-        """Print format of BaseNode
+        """Print format of BaseNode.
 
         Returns:
-            (str)
+            Print format of BaseNode
         """
         class_name = self.__class__.__name__
         node_dict = self.describe(exclude_prefix="_")
@@ -723,18 +782,18 @@ class BaseNode:
         return f"{class_name}({node_description})"
 
     def __rshift__(self: T, other: T) -> None:
-        """Set children using >> bitshift operator for self >> children (other)
+        """Set children using >> bitshift operator for self >> children (other).
 
         Args:
-            other (Self): other node, children
+            other: other node, children
         """
         other.parent = self
 
     def __lshift__(self: T, other: T) -> None:
-        """Set parent using << bitshift operator for self << parent (other)
+        """Set parent using << bitshift operator for self << parent (other).
 
         Args:
-            other (Self): other node, parent
+            other: other node, parent
         """
         self.parent = other
 
@@ -742,18 +801,18 @@ class BaseNode:
         """Iterate through child nodes
 
         Returns:
-            (Self): child node
+            Iterable of child node(s)
         """
         yield from self.children  # type: ignore
 
     def __contains__(self, other_node: T) -> bool:
-        """Check if child node exists
+        """Check if child node exists.
 
         Args:
-            other_node (T): child node
+            other_node: child node
 
         Returns:
-            (bool)
+            Indicator if other node is child of current node
         """
         return other_node in self.children
 
