@@ -1,49 +1,51 @@
 from __future__ import annotations
 
-from typing import Any, List, Optional, Tuple, TypeVar, Union
+from typing import Any, TypeVar
 
-from bigtree.node.node import Node
-from bigtree.utils.exceptions import CorruptedTreeError, LoopError, TreeError
+from bigtree.globals import ASSERTIONS
+from bigtree.node import node
+from bigtree.utils import exceptions
 
 
-class BinaryNode(Node):
+class BinaryNode(node.Node):
     """
     BinaryNode is an extension of Node, and is able to extend to any Python class for Binary Tree implementation.
     Nodes can have attributes if they are initialized from `BinaryNode`, *dictionary*, or *pandas DataFrame*.
 
-    BinaryNode can be linked to each other with `children`, `left`, or `right` setter methods.
-    If initialized with `children`, it must be length 2, denoting left and right child.
+    BinaryNode can be linked to each other with `children`, `left`, or `right` setter methods. If initialized with
+    `children`, it must be length 2, denoting left and right child.
 
-    >>> from bigtree import BinaryNode, print_tree
-    >>> a = BinaryNode(1)
-    >>> b = BinaryNode(2)
-    >>> c = BinaryNode(3)
-    >>> d = BinaryNode(4)
-    >>> a.children = [b, c]
-    >>> b.right = d
-    >>> print_tree(a)
-    1
-    ├── 2
-    │   └── 4
-    └── 3
+    Examples:
+        >>> from bigtree import BinaryNode, print_tree
+        >>> a = BinaryNode(1)
+        >>> b = BinaryNode(2)
+        >>> c = BinaryNode(3)
+        >>> d = BinaryNode(4)
+        >>> a.children = [b, c]
+        >>> b.right = d
+        >>> print_tree(a)
+        1
+        ├── 2
+        │   └── 4
+        └── 3
 
-    Directly passing `left`, `right`, or `children` argument.
+        Directly passing `left`, `right`, or `children` argument.
 
-    >>> from bigtree import BinaryNode
-    >>> d = BinaryNode(4)
-    >>> c = BinaryNode(3)
-    >>> b = BinaryNode(2, right=d)
-    >>> a = BinaryNode(1, children=[b, c])
+        >>> from bigtree import BinaryNode
+        >>> d = BinaryNode(4)
+        >>> c = BinaryNode(3)
+        >>> b = BinaryNode(2, right=d)
+        >>> a = BinaryNode(1, children=[b, c])
 
     **BinaryNode Creation**
 
-    Node can be created by instantiating a `BinaryNode` class or by using a *dictionary*.
-    If node is created with dictionary, all keys of dictionary will be stored as class attributes.
+    Node can be created by instantiating a `BinaryNode` class or by using a *dictionary*. If node is created with
+    dictionary, all keys of dictionary will be stored as class attributes.
 
-    >>> from bigtree import BinaryNode
-    >>> a = BinaryNode.from_dict({"name": "1"})
-    >>> a
-    BinaryNode(name=1, val=1)
+        >>> from bigtree import BinaryNode
+        >>> a = BinaryNode.from_dict({"name": "1"})
+        >>> a
+        BinaryNode(name=1, val=1)
 
     **BinaryNode Attributes**
 
@@ -60,21 +62,21 @@ class BinaryNode(Node):
 
     def __init__(
         self,
-        name: Union[str, int] = "",
-        left: Optional[T] = None,
-        right: Optional[T] = None,
-        parent: Optional[T] = None,
-        children: Optional[List[Optional[T]]] = None,
+        name: str | int = "",
+        left: T | None = None,
+        right: T | None = None,
+        parent: T | None = None,
+        children: list[T | None] | None = None,
         **kwargs: Any,
     ):
         try:
-            self.val: Union[str, int] = int(name)
+            self.val: str | int = int(name)
         except ValueError:
             self.val = str(name)
         self.name = str(name)
         self._sep = "/"
-        self.__parent: Optional[T] = None
-        self.__children: List[Optional[T]] = [None, None]
+        self.__parent: T | None = None
+        self.__children: list[T | None] = [None, None]
         if not children:
             children = []
         if len(children):
@@ -102,46 +104,46 @@ class BinaryNode(Node):
 
     @property
     def left(self: T) -> T:
-        """Get left children
+        """Get left children.
 
         Returns:
-            (Self)
+            Left child
         """
         return self.__children[0]
 
     @left.setter
-    def left(self: T, left_child: Optional[T]) -> None:
-        """Set left children
+    def left(self: T, left_child: T | None) -> None:
+        """Set left children.
 
         Args:
-            left_child (Optional[Self]): left child
+            left_child: left child
         """
         self.children = [left_child, self.right]  # type: ignore
 
     @property
     def right(self: T) -> T:
-        """Get right children
+        """Get right children.
 
         Returns:
-            (Self)
+            Right child
         """
         return self.__children[1]
 
     @right.setter
-    def right(self: T, right_child: Optional[T]) -> None:
-        """Set right children
+    def right(self: T, right_child: T | None) -> None:
+        """Set right children.
 
         Args:
-            right_child (Optional[Self]): right child
+            right_child: right child
         """
         self.children = [self.left, right_child]  # type: ignore
 
     @staticmethod
-    def __check_parent_type(new_parent: Optional[T]) -> None:
-        """Check parent type
+    def __check_parent_type(new_parent: T | None) -> None:
+        """Check parent type.
 
         Args:
-            new_parent (Optional[Self]): parent node
+            new_parent: parent node
         """
         if not (isinstance(new_parent, BinaryNode) or new_parent is None):
             raise TypeError(
@@ -149,23 +151,24 @@ class BinaryNode(Node):
             )
 
     @property
-    def parent(self: T) -> Optional[T]:
-        """Get parent node
+    def parent(self: T) -> T | None:
+        """Get parent node.
 
         Returns:
-            (Optional[Self])
+            Parent node, none if the node is root
         """
         return self.__parent
 
     @parent.setter
-    def parent(self: T, new_parent: Optional[T]) -> None:
-        """Set parent node
+    def parent(self: T, new_parent: T | None) -> None:
+        """Set parent node.
 
         Args:
-            new_parent (Optional[Self]): parent node
+            new_parent: parent node
         """
-        self.__check_parent_type(new_parent)
-        self._BaseNode__check_parent_loop(new_parent)  # type: ignore
+        if ASSERTIONS:
+            self.__check_parent_type(new_parent)
+            self._BaseNode__check_parent_loop(new_parent)  # type: ignore
 
         current_parent = self.parent
         current_child_idx = None
@@ -178,7 +181,7 @@ class BinaryNode(Node):
                 if not any(
                     child is self for child in current_parent.children
                 ):  # pragma: no cover
-                    raise CorruptedTreeError(
+                    raise exceptions.CorruptedTreeError(
                         "Error setting parent: Node does not exist as children of its parent"
                     )
                 current_child_idx = current_parent.__children.index(self)
@@ -193,7 +196,9 @@ class BinaryNode(Node):
                         new_parent.__children[child_idx] = self
                         inserted = True
                 if not inserted:
-                    raise TreeError(f"Parent {new_parent} already has 2 children")
+                    raise exceptions.TreeError(
+                        f"Parent {new_parent} already has 2 children"
+                    )
 
             self.__post_assign_parent(new_parent)
 
@@ -207,36 +212,32 @@ class BinaryNode(Node):
             self.__parent = current_parent
             if current_child_idx is not None:
                 current_parent.__children[current_child_idx] = self
-            raise TreeError(exc_info)
+            raise exceptions.TreeError(exc_info) from None
 
-    def __pre_assign_parent(self: T, new_parent: Optional[T]) -> None:
-        """Custom method to check before attaching parent
-        Can be overridden with `_BinaryNode__pre_assign_parent()`
+    def __pre_assign_parent(self: T, new_parent: T | None) -> None:
+        """Custom method to check before attaching parent. Can be overridden with `_BinaryNode__pre_assign_parent()`.
 
         Args:
-            new_parent (Optional[Self]): new parent to be added
+            new_parent: new parent to be added
         """
         pass
 
-    def __post_assign_parent(self: T, new_parent: Optional[T]) -> None:
-        """Custom method to check after attaching parent
-        Can be overridden with `_BinaryNode__post_assign_parent()`
+    def __post_assign_parent(self: T, new_parent: T | None) -> None:
+        """Custom method to check after attaching parent. Can be overridden with `_BinaryNode__post_assign_parent()`.
 
         Args:
-            new_parent (Optional[Self]): new parent to be added
+            new_parent: new parent to be added
         """
         pass
 
-    def __check_children_type(
-        self: T, new_children: List[Optional[T]]
-    ) -> List[Optional[T]]:
-        """Check child type
+    def __check_children_type(self: T, new_children: list[T | None]) -> list[T | None]:
+        """Check child type.
 
         Args:
-            new_children (List[Optional[Self]]): child node
+            new_children: child node
 
         Returns:
-            (List[Optional[Self]])
+            Child nodes, initialise child to None if no new children defined
         """
         if not len(new_children):
             new_children = [None, None]
@@ -244,11 +245,11 @@ class BinaryNode(Node):
             raise ValueError("Children input must have length 2")
         return new_children
 
-    def __check_children_loop(self: T, new_children: List[Optional[T]]) -> None:
-        """Check child loop
+    def __check_children_loop(self: T, new_children: list[T | None]) -> None:
+        """Check child loop.
 
         Args:
-            new_children (List[Optional[Self]]): child node
+            new_children: child node
         """
         seen_children = []
         for new_child in new_children:
@@ -260,40 +261,43 @@ class BinaryNode(Node):
 
             # Check for loop and tree structure
             if new_child is self:
-                raise LoopError("Error setting child: Node cannot be child of itself")
+                raise exceptions.LoopError(
+                    "Error setting child: Node cannot be child of itself"
+                )
             if any(child is new_child for child in self.ancestors):
-                raise LoopError(
+                raise exceptions.LoopError(
                     "Error setting child: Node cannot be ancestor of itself"
                 )
 
             # Check for duplicate children
             if new_child is not None:
                 if id(new_child) in seen_children:
-                    raise TreeError(
+                    raise exceptions.TreeError(
                         "Error setting child: Node cannot be added multiple times as a child"
                     )
                 else:
                     seen_children.append(id(new_child))
 
     @property
-    def children(self: T) -> Tuple[T, ...]:
-        """Get child nodes
+    def children(self: T) -> tuple[T, ...]:
+        """Get child nodes.
 
         Returns:
-            (Tuple[Optional[Self]])
+            Child nodes
         """
         return tuple(self.__children)
 
     @children.setter
-    def children(self: T, _new_children: List[Optional[T]]) -> None:
-        """Set child nodes
+    def children(self: T, _new_children: list[T | None]) -> None:
+        """Set child nodes.
 
         Args:
-            _new_children (List[Optional[Self]]): child node
+            _new_children: child node
         """
         self._BaseNode__check_children_type(_new_children)  # type: ignore
         new_children = self.__check_children_type(_new_children)
-        self.__check_children_loop(new_children)
+        if ASSERTIONS:
+            self.__check_children_loop(new_children)
 
         current_new_children = {
             new_child: (
@@ -339,59 +343,58 @@ class BinaryNode(Node):
             for child in current_children:
                 if child:
                     child.__parent = self
-            raise TreeError(exc_info)
+            raise exceptions.TreeError(exc_info) from None
 
     @children.deleter
     def children(self) -> None:
-        """Delete child node(s)"""
+        """Delete child node(s)."""
         for child in self.children:
             if child is not None:
                 child.parent.__children.remove(child)  # type: ignore
                 child.__parent = None
 
-    def __pre_assign_children(self: T, new_children: List[Optional[T]]) -> None:
-        """Custom method to check before attaching children
-        Can be overridden with `_BinaryNode__pre_assign_children()`
+    def __pre_assign_children(self: T, new_children: list[T | None]) -> None:
+        """Custom method to check before attaching children. Can be overridden with `_BinaryNode__pre_assign_children()`.
 
         Args:
-            new_children (List[Optional[Self]]): new children to be added
+            new_children: new children to be added
         """
         pass
 
-    def __post_assign_children(self: T, new_children: List[Optional[T]]) -> None:
-        """Custom method to check after attaching children
-        Can be overridden with `_BinaryNode__post_assign_children()`
+    def __post_assign_children(self: T, new_children: list[T | None]) -> None:
+        """Custom method to check after attaching children. Can be overridden with `_BinaryNode__post_assign_children()`.
 
         Args:
-            new_children (List[Optional[Self]]): new children to be added
+            new_children: new children to be added
         """
         pass
 
     @property
     def is_leaf(self) -> bool:
-        """Get indicator if self is leaf node
+        """Get indicator if self is leaf node.
 
         Returns:
-            (bool)
+            Indicator if node is leaf node
         """
         return not len([child for child in self.children if child])
 
     def sort(self, **kwargs: Any) -> None:
-        """Sort children, possible keyword arguments include ``key=lambda node: node.val``, ``reverse=True``
+        """Sort children, possible keyword arguments include ``key=lambda node: node.val``, ``reverse=True``.
 
-        >>> from bigtree import BinaryNode, print_tree
-        >>> a = BinaryNode(1)
-        >>> c = BinaryNode(3, parent=a)
-        >>> b = BinaryNode(2, parent=a)
-        >>> print_tree(a)
-        1
-        ├── 3
-        └── 2
-        >>> a.sort(key=lambda node: node.val)
-        >>> print_tree(a)
-        1
-        ├── 2
-        └── 3
+        Examples:
+            >>> from bigtree import BinaryNode, print_tree
+            >>> a = BinaryNode(1)
+            >>> c = BinaryNode(3, parent=a)
+            >>> b = BinaryNode(2, parent=a)
+            >>> print_tree(a)
+            1
+            ├── 3
+            └── 2
+            >>> a.sort(key=lambda node: node.val)
+            >>> print_tree(a)
+            1
+            ├── 2
+            └── 3
         """
         children = [child for child in self.children if child]
         if len(children) == 2:
@@ -399,10 +402,10 @@ class BinaryNode(Node):
             self.__children = children  # type: ignore
 
     def __repr__(self) -> str:
-        """Print format of BinaryNode
+        """Print format of BinaryNode.
 
         Returns:
-            (str)
+            Print format of BinaryNode
         """
         class_name = self.__class__.__name__
         node_dict = self.describe(exclude_prefix="_", exclude_attributes=[])
