@@ -2,23 +2,9 @@ import unittest
 
 import pytest
 
-from bigtree.node.node import Node
-from bigtree.tree.search import (
-    find,
-    find_attr,
-    find_attrs,
-    find_child,
-    find_child_by_name,
-    find_children,
-    find_full_path,
-    find_name,
-    find_names,
-    find_path,
-    find_paths,
-    find_relative_path,
-    findall,
-)
-from bigtree.utils.exceptions import SearchError
+from bigtree.node import node
+from bigtree.tree import search
+from bigtree.utils import exceptions
 from tests.test_constants import Constants
 
 
@@ -35,14 +21,14 @@ class TestSearch(unittest.TestCase):
         +-- c (age=60)
             +-- f (age=38)
         """
-        self.a = Node("a", age=90)
-        self.b = Node("b", age=65)
-        self.c = Node("c", age=60)
-        self.d = Node("d", age=40)
-        self.e = Node("e", age=35)
-        self.f = Node("f", age=38)
-        self.g = Node("g", age=10)
-        self.h = Node("h", age=6)
+        self.a = node.Node("a", age=90)
+        self.b = node.Node("b", age=65)
+        self.c = node.Node("c", age=60)
+        self.d = node.Node("d", age=40)
+        self.e = node.Node("e", age=35)
+        self.f = node.Node("f", age=38)
+        self.g = node.Node("g", age=10)
+        self.h = node.Node("h", age=6)
 
         self.b.parent = self.a
         self.c.parent = self.a
@@ -63,69 +49,73 @@ class TestSearch(unittest.TestCase):
         self.h = None
 
     def test_find_all(self):
-        actual = findall(self.a, lambda node: node.age >= 60)
+        actual = search.findall(self.a, lambda _node: _node.age >= 60)
         expected = (self.a, self.b, self.c)
         assert (
             actual == expected
         ), f"Expected find_all to return {expected}, received {actual}"
 
-        actual = findall(self.a, lambda node: node.age >= 30)
+        actual = search.findall(self.a, lambda _node: _node.age >= 30)
         expected = (self.a, self.b, self.d, self.e, self.c, self.f)
         assert (
             actual == expected
         ), f"Expected find_all to return {expected}, received {actual}"
 
     def test_find_all_descendant(self):
-        actual = findall(self.b, lambda node: node.age >= 60)
+        actual = search.findall(self.b, lambda _node: _node.age >= 60)
         expected = (self.b,)
         assert (
             actual == expected
         ), f"Expected find_all to return {expected}, received {actual}"
 
     def test_find_all_max_depth(self):
-        actual = findall(self.a, lambda node: node.age >= 30, max_depth=2)
+        actual = search.findall(self.a, lambda _node: _node.age >= 30, max_depth=2)
         expected = (self.a, self.b, self.c)
         assert (
             actual == expected
         ), f"Expected find_all to return {expected}, received {actual}"
 
     def test_find_all_max_count_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            findall(self.a, lambda node: node.age >= 30, max_depth=2, max_count=2)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.findall(
+                self.a, lambda _node: _node.age >= 30, max_depth=2, max_count=2
+            )
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_LESS_THAN_N_ELEMENT.format(count=2)
         )
 
     def test_find_all_min_count_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            findall(self.a, lambda node: node.age >= 30, max_depth=2, min_count=4)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.findall(
+                self.a, lambda _node: _node.age >= 30, max_depth=2, min_count=4
+            )
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_MORE_THAN_N_ELEMENT.format(count=4)
         )
 
     def test_find(self):
-        actual = find(self.a, lambda node: node.age == 60)
+        actual = search.find(self.a, lambda _node: _node.age == 60)
         expected = self.c
         assert (
             actual == expected
         ), f"Expected find to return {expected}, received {actual}"
 
-        actual = find(self.a, lambda node: node.age == 5)
+        actual = search.find(self.a, lambda _node: _node.age == 5)
         expected = None
         assert (
             actual == expected
         ), f"Expected find to return {expected}, received {actual}"
 
     def test_find_descendant(self):
-        actual = find(self.b, lambda node: node.age == 60)
+        actual = search.find(self.b, lambda _node: _node.age == 60)
         expected = None
         assert (
             actual == expected
         ), f"Expected find to return {expected}, received {actual}"
 
     def test_find_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            find(self.a, lambda node: node.age > 5)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.find(self.a, lambda _node: _node.age > 5)
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_LESS_THAN_N_ELEMENT.format(count=1)
         )
@@ -144,7 +134,7 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_name(self.a, input_)
+            actual = search.find_name(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_name to return {expected}, received {actual}"
@@ -163,39 +153,58 @@ class TestSearch(unittest.TestCase):
             (),
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_names(self.a, input_)
+            actual = search.find_names(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_names to return {expected}, received {actual}"
 
     def test_find_relative_path_current_position(self):
         nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
-        for node in nodes:
-            expected = node
-            actual = find_relative_path(node, ".")
-            assert actual == (
-                expected,
+        for _node in nodes:
+            expected = _node
+            actual = search.find_relative_path(_node, ".")
+            assert (
+                actual == expected
             ), f"Expected find_relative_path to return {expected}, received {actual}"
 
-    def test_find_relative_path_current_position_multiple(self):
+    def test_find_relative_path_max_count_error(self):
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.find_relative_path(self.a, "*")
+        assert str(exc_info.value).startswith(
+            Constants.ERROR_SEARCH_LESS_THAN_N_ELEMENT.format(count=1)
+        )
+
+    def test_find_relative_path_no_results(self):
+        assert not search.find_relative_path(self.a, "b/d/*")
+
+    def test_find_relative_paths_current_position(self):
         nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
-        for node in nodes:
-            expected = node
-            actual = find_relative_path(node, "./././././.")
+        for _node in nodes:
+            expected = _node
+            actual = search.find_relative_paths(_node, ".")
             assert actual == (
                 expected,
-            ), f"Expected find_relative_path to return {expected}, received {actual}"
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_parent_position(self):
+    def test_find_relative_paths_current_position_multiple(self):
+        nodes = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
+        for _node in nodes:
+            expected = _node
+            actual = search.find_relative_paths(_node, "./././././.")
+            assert actual == (
+                expected,
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
+
+    def test_find_relative_paths_parent_position(self):
         inputs = [self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         expected_ans = [self.a, self.a, self.b, self.b, self.c, self.e, self.e]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_relative_path(input_, "..")
+            actual = search.find_relative_paths(input_, "..")
             assert actual == (
                 expected,
-            ), f"Expected find_relative_path to return {expected}, received {actual}"
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_wildcard(self):
+    def test_find_relative_paths_wildcard(self):
         inputs = ["*", "b/*", "c/*", "b/e/*"]
         expected_ans = [
             (self.b, self.c),
@@ -204,12 +213,12 @@ class TestSearch(unittest.TestCase):
             (self.g, self.h),
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_relative_path(self.a, input_)
+            actual = search.find_relative_paths(self.a, input_)
             assert (
                 actual == expected
-            ), f"Expected find_relative_path to return {expected}, received {actual}"
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_wildcard_parent_path(self):
+    def test_find_relative_paths_wildcard_parent_path(self):
         h2 = self.h.copy()
         h2.parent = self.f
         inputs_list = [
@@ -223,12 +232,12 @@ class TestSearch(unittest.TestCase):
         ]
         for inputs in inputs_list:
             for input_, expected in zip(inputs[1], expected_ans):
-                actual = find_relative_path(inputs[0], input_)
+                actual = search.find_relative_paths(inputs[0], input_)
                 assert (
                     actual == expected
-                ), f"Expected find_relative_path to return {expected}, received {actual}"
+                ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_sep_leading(self):
+    def test_find_relative_paths_sep_leading(self):
         inputs = [
             "/a",
             "/a/b",
@@ -241,41 +250,71 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_relative_path(self.b, input_)
+            actual = search.find_relative_paths(self.b, input_)
             assert actual == (
                 expected,
-            ), f"Expected find_relative_path to return {expected}, received {actual}"
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_sep_trailing(self):
+    def test_find_relative_paths_sep_leading_wildcard(self):
+        inputs = ["/*", "/a/b/*", "/a/c/*", "/a/b/e/*"]
+        expected_ans = [
+            (self.b, self.c),
+            (self.d, self.e),
+            (self.f,),
+            (self.g, self.h),
+        ]
+        for input_, expected in zip(inputs, expected_ans):
+            actual = search.find_relative_paths(self.b, input_)
+            assert (
+                actual == expected
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
+
+    def test_find_relative_paths_sep_trailing(self):
         inputs = [self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         expected_ans = [self.a, self.a, self.b, self.b, self.c, self.e, self.e]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_relative_path(input_, "../")
+            actual = search.find_relative_paths(input_, "../")
             assert actual == (
                 expected,
-            ), f"Expected find_relative_path to return {expected}, received {actual}"
+            ), f"Expected find_relative_paths to return {expected}, received {actual}"
 
-    def test_find_relative_path_wrong_node_error(self):
+    def test_find_relative_paths_wrong_root_error(self):
+        inputs = ["/b/a", "/c"]
+        for input_ in inputs:
+            with pytest.raises(ValueError) as exc_info:
+                search.find_relative_paths(self.a, input_)
+            assert str(
+                exc_info.value
+            ) == Constants.ERROR_SEARCH_FULL_PATH_INVALID_ROOT.format(
+                path_name=input_, root_name="a"
+            )
+
+    def test_find_relative_paths_wrong_node_error(self):
         inputs = [
             ("a/e", "a"),
             ("b/f", "f"),
             ("b/e/i", "i"),
         ]
         for input_, component_ in inputs:
-            with pytest.raises(SearchError) as exc_info:
-                find_relative_path(self.a, input_)
+            with pytest.raises(exceptions.SearchError) as exc_info:
+                search.find_relative_paths(self.a, input_)
             assert str(
                 exc_info.value
             ) == Constants.ERROR_SEARCH_RELATIVE_INVALID_NODE.format(
                 component=component_
             )
 
-    def test_find_relative_path_wrong_path_error(self):
-        inputs_list = [(self.a, ["../"]), (self.b, ["../../"])]
+    def test_find_relative_paths_wrong_path_error(self):
+        inputs_list = [
+            (self.a, ["../"]),
+            (self.b, ["../../"]),
+            (self.a, ["/../"]),
+            (self.b, ["/.."]),
+        ]
         for inputs in inputs_list:
             for input_ in inputs[1]:
-                with pytest.raises(SearchError) as exc_info:
-                    find_relative_path(inputs[0], input_)
+                with pytest.raises(exceptions.SearchError) as exc_info:
+                    search.find_relative_paths(inputs[0], input_)
         assert str(exc_info.value) == Constants.ERROR_SEARCH_RELATIVE_INVALID_PATH
 
     def test_find_full_path(self):
@@ -291,7 +330,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_full_path(self.a, input_)
+            actual = search.find_full_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_full_path to return {expected}, received {actual}"
@@ -309,7 +348,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_full_path(self.a, input_)
+            actual = search.find_full_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_full_path to return {expected}, received {actual}"
@@ -327,7 +366,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_full_path(self.a, input_)
+            actual = search.find_full_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_full_path to return {expected}, received {actual}"
@@ -339,7 +378,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [None, None]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_full_path(self.a, input_)
+            actual = search.find_full_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_full_path to return {expected}, received {actual}"
@@ -354,7 +393,7 @@ class TestSearch(unittest.TestCase):
         expected_ans = [None, None, None]
         for input_, expected in zip(inputs, expected_ans):
             with pytest.raises(ValueError) as exc_info:
-                find_full_path(self.a, input_)
+                search.find_full_path(self.a, input_)
             assert str(
                 exc_info.value
             ) == Constants.ERROR_SEARCH_FULL_PATH_INVALID_ROOT.format(
@@ -385,7 +424,7 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_path(self.a, input_)
+            actual = search.find_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_path to return {expected}, received {actual}"
@@ -404,7 +443,7 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_path(self.a, input_)
+            actual = search.find_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_path to return {expected}, received {actual}"
@@ -422,7 +461,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_path(self.a, input_)
+            actual = search.find_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_path to return {expected}, received {actual}"
@@ -440,7 +479,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_path(self.a, input_)
+            actual = search.find_path(self.a, input_)
             assert (
                 actual == expected
             ), f"Expected find_name to return {expected}, received {actual}"
@@ -458,7 +497,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_paths(self.a, input_)
+            actual = search.find_paths(self.a, input_)
             assert actual == (
                 expected,
             ), f"Expected find_paths to return {expected}, received {actual}"
@@ -476,7 +515,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_paths(self.a, input_)
+            actual = search.find_paths(self.a, input_)
             assert actual == (
                 expected,
             ), f"Expected find_paths to return {expected}, received {actual}"
@@ -494,7 +533,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_paths(self.a, input_)
+            actual = search.find_paths(self.a, input_)
             assert actual == (
                 expected,
             ), f"Expected find_paths to return {expected}, received {actual}"
@@ -512,7 +551,7 @@ class TestSearch(unittest.TestCase):
         ]
         expected_ans = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_paths(self.a, input_)
+            actual = search.find_paths(self.a, input_)
             assert actual == (
                 expected,
             ), f"Expected find_paths to return {expected}, received {actual}"
@@ -531,7 +570,7 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_attr(self.a, "name", input_)
+            actual = search.find_attr(self.a, "name", input_)
             assert (
                 actual == expected
             ), f"Expected find_attr to return {expected}, received {actual}"
@@ -549,7 +588,7 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_attr(self.a, "age", input_)
+            actual = search.find_attr(self.a, "age", input_)
             assert (
                 actual == expected
             ), f"Expected find_attr to return {expected}, received {actual}"
@@ -568,7 +607,7 @@ class TestSearch(unittest.TestCase):
             (),
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_attrs(self.a, "name", input_)
+            actual = search.find_attrs(self.a, "name", input_)
             assert (
                 actual == expected
             ), f"Expected find_attrs to return {expected}, received {actual}"
@@ -586,7 +625,7 @@ class TestSearch(unittest.TestCase):
             (),
         ]
         for input_, expected in zip(inputs, expected_ans):
-            actual = find_attrs(self.a, "age", input_)
+            actual = search.find_attrs(self.a, "age", input_)
             assert (
                 actual == expected
             ), f"Expected find_attrs to return {expected}, received {actual}"
@@ -603,12 +642,12 @@ class TestSearch(unittest.TestCase):
             (),
             (),
         ]
-        for idx, input in enumerate(inputs):
-            actual = find_children(input, lambda node: node.age > 1)
+        for idx, input_ in enumerate(inputs):
+            actual = search.find_children(input_, lambda _node: _node.age > 1)
             expected = expected_ans[idx]
             assert (
                 actual == expected
-            ), f"Expected find_children to return {expected}, received {actual} for input {input}"
+            ), f"Expected find_children to return {expected}, received {actual} for input {input_}"
 
     def test_find_children_condition(self):
         inputs = [self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h]
@@ -623,27 +662,27 @@ class TestSearch(unittest.TestCase):
             (),
         ]
         for idx, input_ in enumerate(inputs):
-            actual = find_children(
+            actual = search.find_children(
                 input_,
-                lambda node: any(
-                    node.node_name.startswith(_name) for _name in ["b", "e", "f", "g"]
+                lambda _node: any(
+                    _node.node_name.startswith(_name) for _name in ["b", "e", "f", "g"]
                 ),
             )
             expected = expected_ans[idx]
             assert (
                 actual == expected
-            ), f"Expected find_children to return {expected}, received {actual} for input {input}"
+            ), f"Expected find_children to return {expected}, received {actual} for input {input_}"
 
     def test_find_children_max_count_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            find_children(self.a, lambda node: node.age >= 30, max_count=1)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.find_children(self.a, lambda _node: _node.age >= 30, max_count=1)
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_LESS_THAN_N_ELEMENT.format(count=1)
         )
 
     def test_find_children_min_count_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            find_children(self.a, lambda node: node.age >= 30, min_count=3)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.find_children(self.a, lambda _node: _node.age >= 30, min_count=3)
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_MORE_THAN_N_ELEMENT.format(count=3)
         )
@@ -661,20 +700,20 @@ class TestSearch(unittest.TestCase):
             None,
         ]
         for idx, input_ in enumerate(inputs):
-            actual = find_child(
+            actual = search.find_child(
                 input_,
-                lambda node: any(
-                    node.node_name.startswith(_name) for _name in ["b", "e", "f", "g"]
+                lambda _node: any(
+                    _node.node_name.startswith(_name) for _name in ["b", "e", "f", "g"]
                 ),
             )
             expected = expected_ans[idx]
             assert (
                 actual == expected
-            ), f"Expected find_children to return {expected}, received {actual} for input {input}"
+            ), f"Expected find_children to return {expected}, received {actual} for input {input_}"
 
     def test_find_child_error(self):
-        with pytest.raises(SearchError) as exc_info:
-            find_child(self.a, lambda node: node.age > 5)
+        with pytest.raises(exceptions.SearchError) as exc_info:
+            search.find_child(self.a, lambda _node: _node.age > 5)
         assert str(exc_info.value).startswith(
             Constants.ERROR_SEARCH_LESS_THAN_N_ELEMENT.format(count=1)
         )
@@ -694,7 +733,7 @@ class TestSearch(unittest.TestCase):
         ]
         for idx1, input_1 in enumerate(inputs1):
             for idx2, input_2 in enumerate(inputs2):
-                actual = find_child_by_name(input_1, input_2)
+                actual = search.find_child_by_name(input_1, input_2)
                 expected = expected_ans[idx1][idx2]
                 assert (
                     actual == expected
