@@ -1,26 +1,17 @@
-from bigtree.utils.iterators import (
-    dag_iterator,
-    inorder_iter,
-    levelorder_iter,
-    levelordergroup_iter,
-    postorder_iter,
-    preorder_iter,
-    zigzag_iter,
-    zigzaggroup_iter,
-)
+from bigtree.utils import iterators
 
 
 class TestPreOrderIter:
     @staticmethod
     def test_preorder_iter(tree_node):
         expected = ["a", "b", "d", "e", "g", "h", "c", "f"]
-        actual = [node.node_name for node in preorder_iter(tree_node)]
+        actual = [node.node_name for node in iterators.preorder_iter(tree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_preorder_iter2(tree_node2):
         expected = ["a", "b", "d", "g", "e", "h", "i", "c", "f"]
-        actual = [node.node_name for node in preorder_iter(tree_node2)]
+        actual = [node.node_name for node in iterators.preorder_iter(tree_node2)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -28,7 +19,7 @@ class TestPreOrderIter:
         expected = ["a", "d", "e", "g", "f"]
         actual = [
             node.node_name
-            for node in preorder_iter(
+            for node in iterators.preorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
@@ -40,7 +31,7 @@ class TestPreOrderIter:
         expected = ["g", "h"]
         actual = [
             node.node_name
-            for node in preorder_iter(
+            for node in iterators.preorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
@@ -52,7 +43,7 @@ class TestPreOrderIter:
         expected = ["a", "b", "d", "c", "f"]
         actual = [
             node.node_name
-            for node in preorder_iter(
+            for node in iterators.preorder_iter(
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
@@ -61,7 +52,9 @@ class TestPreOrderIter:
     @staticmethod
     def test_preorder_iter_max_depth(tree_node):
         expected = ["a", "b", "d", "e", "c", "f"]
-        actual = [node.node_name for node in preorder_iter(tree_node, max_depth=3)]
+        actual = [
+            node.node_name for node in iterators.preorder_iter(tree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
@@ -69,13 +62,13 @@ class TestPostOrderIter:
     @staticmethod
     def test_postorder_iter(tree_node):
         expected = ["d", "g", "h", "e", "b", "f", "c", "a"]
-        actual = [node.node_name for node in postorder_iter(tree_node)]
+        actual = [node.node_name for node in iterators.postorder_iter(tree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter2(tree_node2):
         expected = ["g", "d", "h", "i", "e", "b", "f", "c", "a"]
-        actual = [node.node_name for node in postorder_iter(tree_node2)]
+        actual = [node.node_name for node in iterators.postorder_iter(tree_node2)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -83,7 +76,7 @@ class TestPostOrderIter:
         expected = ["d", "g", "e", "f", "a"]
         actual = [
             node.node_name
-            for node in postorder_iter(
+            for node in iterators.postorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
@@ -95,7 +88,7 @@ class TestPostOrderIter:
         expected = ["g", "h"]
         actual = [
             node.node_name
-            for node in postorder_iter(
+            for node in iterators.postorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
@@ -107,7 +100,7 @@ class TestPostOrderIter:
         expected = ["d", "b", "f", "c", "a"]
         actual = [
             node.node_name
-            for node in postorder_iter(
+            for node in iterators.postorder_iter(
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
@@ -116,7 +109,9 @@ class TestPostOrderIter:
     @staticmethod
     def test_postorder_iter_max_depth(tree_node):
         expected = ["d", "e", "b", "f", "c", "a"]
-        actual = [node.node_name for node in postorder_iter(tree_node, max_depth=3)]
+        actual = [
+            node.node_name for node in iterators.postorder_iter(tree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
@@ -124,13 +119,13 @@ class TestLevelOrderIter:
     @staticmethod
     def test_levelorder_iter(tree_node):
         expected = ["a", "b", "c", "d", "e", "f", "g", "h"]
-        actual = [node.node_name for node in levelorder_iter(tree_node)]
+        actual = [node.node_name for node in iterators.levelorder_iter(tree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter2(tree_node2):
         expected = ["a", "b", "c", "d", "e", "f", "g", "h", "i"]
-        actual = [node.node_name for node in levelorder_iter(tree_node2)]
+        actual = [node.node_name for node in iterators.levelorder_iter(tree_node2)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -138,7 +133,7 @@ class TestLevelOrderIter:
         expected = ["a", "d", "e", "f", "g"]
         actual = [
             node.node_name
-            for node in levelorder_iter(
+            for node in iterators.levelorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
@@ -150,7 +145,7 @@ class TestLevelOrderIter:
         expected = ["g", "h"]
         actual = [
             node.node_name
-            for node in levelorder_iter(
+            for node in iterators.levelorder_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
@@ -162,7 +157,7 @@ class TestLevelOrderIter:
         expected = ["a", "b", "c", "d", "f"]
         actual = [
             node.node_name
-            for node in levelorder_iter(
+            for node in iterators.levelorder_iter(
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
@@ -171,7 +166,9 @@ class TestLevelOrderIter:
     @staticmethod
     def test_levelorder_iter_max_depth(tree_node):
         expected = ["a", "b", "c", "d", "e", "f"]
-        actual = [node.node_name for node in levelorder_iter(tree_node, max_depth=3)]
+        actual = [
+            node.node_name for node in iterators.levelorder_iter(tree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
@@ -181,7 +178,7 @@ class TestLevelOrderGroupIter:
         expected = [["a"], ["b", "c"], ["d", "e", "f"], ["g", "h"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(tree_node)
+            for group in iterators.levelordergroup_iter(tree_node)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -190,7 +187,7 @@ class TestLevelOrderGroupIter:
         expected = [["a"], ["b", "c"], ["d", "e", "f"], ["g", "h", "i"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(tree_node2)
+            for group in iterators.levelordergroup_iter(tree_node2)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -199,9 +196,9 @@ class TestLevelOrderGroupIter:
         expected = [["a"], [], ["d", "e", "f"], ["g"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(
+            for group in iterators.levelordergroup_iter(
                 tree_node,
-                filter_condition=lambda x: x.name in ["a", "d", "e", "f", "g"],
+                filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -211,9 +208,9 @@ class TestLevelOrderGroupIter:
         expected = [[], [], [], ["g", "h"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(
+            for group in iterators.levelordergroup_iter(
                 tree_node,
-                filter_condition=lambda x: x.name in ["g", "h"],
+                filter_condition=lambda x: x.node_name in ["g", "h"],
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -223,8 +220,8 @@ class TestLevelOrderGroupIter:
         expected = [["a"], ["b", "c"], ["d", "f"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(
-                tree_node, stop_condition=lambda x: x.name == "e"
+            for group in iterators.levelordergroup_iter(
+                tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -234,7 +231,7 @@ class TestLevelOrderGroupIter:
         expected = [["a"], ["b", "c"], ["d", "e", "f"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(tree_node, max_depth=3)
+            for group in iterators.levelordergroup_iter(tree_node, max_depth=3)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -243,13 +240,13 @@ class TestZigZagIter:
     @staticmethod
     def test_zigzag_iter(tree_node):
         expected = ["a", "c", "b", "d", "e", "f", "h", "g"]
-        actual = [node.node_name for node in zigzag_iter(tree_node)]
+        actual = [node.node_name for node in iterators.zigzag_iter(tree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_zigzag_iter2(tree_node2):
         expected = ["a", "c", "b", "d", "e", "f", "i", "h", "g"]
-        actual = [node.node_name for node in zigzag_iter(tree_node2)]
+        actual = [node.node_name for node in iterators.zigzag_iter(tree_node2)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -257,7 +254,7 @@ class TestZigZagIter:
         expected = ["a", "d", "e", "f", "g"]
         actual = [
             node.node_name
-            for node in zigzag_iter(
+            for node in iterators.zigzag_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
@@ -269,7 +266,7 @@ class TestZigZagIter:
         expected = ["h", "g"]
         actual = [
             node.node_name
-            for node in zigzag_iter(
+            for node in iterators.zigzag_iter(
                 tree_node,
                 filter_condition=lambda x: x.node_name in ["g", "h"],
             )
@@ -281,7 +278,7 @@ class TestZigZagIter:
         expected = ["a", "c", "b", "d", "f"]
         actual = [
             node.node_name
-            for node in zigzag_iter(
+            for node in iterators.zigzag_iter(
                 tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
@@ -290,7 +287,9 @@ class TestZigZagIter:
     @staticmethod
     def test_zigzag_iter_max_depth(tree_node):
         expected = ["a", "c", "b", "d", "e", "f"]
-        actual = [node.node_name for node in zigzag_iter(tree_node, max_depth=3)]
+        actual = [
+            node.node_name for node in iterators.zigzag_iter(tree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
 
@@ -299,7 +298,8 @@ class TestZigZagGroupIter:
     def test_zigzaggroup_iter(tree_node):
         expected = [["a"], ["c", "b"], ["d", "e", "f"], ["h", "g"]]
         actual = [
-            [node.node_name for node in group] for group in zigzaggroup_iter(tree_node)
+            [node.node_name for node in group]
+            for group in iterators.zigzaggroup_iter(tree_node)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -307,7 +307,8 @@ class TestZigZagGroupIter:
     def test_zigzaggroup_iter2(tree_node2):
         expected = [["a"], ["c", "b"], ["d", "e", "f"], ["i", "h", "g"]]
         actual = [
-            [node.node_name for node in group] for group in zigzaggroup_iter(tree_node2)
+            [node.node_name for node in group]
+            for group in iterators.zigzaggroup_iter(tree_node2)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -316,9 +317,9 @@ class TestZigZagGroupIter:
         expected = [["a"], [], ["d", "e", "f"], ["g"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(
+            for group in iterators.zigzaggroup_iter(
                 tree_node,
-                filter_condition=lambda x: x.name in ["a", "d", "e", "f", "g"],
+                filter_condition=lambda x: x.node_name in ["a", "d", "e", "f", "g"],
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -328,9 +329,9 @@ class TestZigZagGroupIter:
         expected = [[], [], [], ["h", "g"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(
+            for group in iterators.zigzaggroup_iter(
                 tree_node,
-                filter_condition=lambda x: x.name in ["g", "h"],
+                filter_condition=lambda x: x.node_name in ["g", "h"],
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -340,8 +341,8 @@ class TestZigZagGroupIter:
         expected = [["a"], ["c", "b"], ["d", "f"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(
-                tree_node, stop_condition=lambda x: x.name == "e"
+            for group in iterators.zigzaggroup_iter(
+                tree_node, stop_condition=lambda x: x.node_name == "e"
             )
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
@@ -351,7 +352,7 @@ class TestZigZagGroupIter:
         expected = [["a"], ["c", "b"], ["d", "e", "f"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(tree_node, max_depth=3)
+            for group in iterators.zigzaggroup_iter(tree_node, max_depth=3)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -372,7 +373,7 @@ class TestDAGIterator:
         ]
         actual = [
             (parent.node_name, child.node_name)
-            for parent, child in dag_iterator(dag_node)
+            for parent, child in iterators.dag_iterator(dag_node)
         ]
         len_expected = 9
         len_actual = len(actual)
@@ -396,7 +397,7 @@ class TestDAGIterator:
         ]
         actual = [
             (parent.node_name, child.node_name)
-            for parent, child in dag_iterator(dag_node_child)
+            for parent, child in iterators.dag_iterator(dag_node_child)
         ]
         len_expected = 9
         len_actual = len(actual)
@@ -410,7 +411,7 @@ class TestBinaryTreeIterator:
     @staticmethod
     def test_preorder_iter(binarytree_node):
         expected = ["1", "2", "4", "8", "5", "3", "6", "7"]
-        actual = [node.node_name for node in preorder_iter(binarytree_node)]
+        actual = [node.node_name for node in iterators.preorder_iter(binarytree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -418,7 +419,7 @@ class TestBinaryTreeIterator:
         expected = ["1", "4", "3", "6"]
         actual = [
             node.node_name
-            for node in preorder_iter(
+            for node in iterators.preorder_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "3", "6", "4"],
             )
@@ -430,7 +431,7 @@ class TestBinaryTreeIterator:
         expected = ["1", "2", "4", "8", "5"]
         actual = [
             node.node_name
-            for node in preorder_iter(
+            for node in iterators.preorder_iter(
                 binarytree_node, stop_condition=lambda x: x.node_name == "3"
             )
         ]
@@ -440,20 +441,21 @@ class TestBinaryTreeIterator:
     def test_preorder_iter_max_depth(binarytree_node):
         expected = ["1", "2", "4", "5", "3", "6", "7"]
         actual = [
-            node.node_name for node in preorder_iter(binarytree_node, max_depth=3)
+            node.node_name
+            for node in iterators.preorder_iter(binarytree_node, max_depth=3)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_postorder_iter(binarytree_node):
         expected = ["8", "4", "5", "2", "6", "7", "3", "1"]
-        actual = [node.node_name for node in postorder_iter(binarytree_node)]
+        actual = [node.node_name for node in iterators.postorder_iter(binarytree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_levelorder_iter(binarytree_node):
         expected = ["1", "2", "3", "4", "5", "6", "7", "8"]
-        actual = [node.node_name for node in levelorder_iter(binarytree_node)]
+        actual = [node.node_name for node in iterators.levelorder_iter(binarytree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -461,7 +463,7 @@ class TestBinaryTreeIterator:
         expected = ["1", "3", "4", "6", "7"]
         actual = [
             node.node_name
-            for node in levelorder_iter(
+            for node in iterators.levelorder_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
@@ -472,7 +474,8 @@ class TestBinaryTreeIterator:
     def test_levelorder_iter_max_depth(binarytree_node):
         expected = ["1", "2", "3", "4", "5", "6", "7"]
         actual = [
-            node.node_name for node in levelorder_iter(binarytree_node, max_depth=3)
+            node.node_name
+            for node in iterators.levelorder_iter(binarytree_node, max_depth=3)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -481,7 +484,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["2", "3"], ["4", "5", "6", "7"], ["8"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(binarytree_node)
+            for group in iterators.levelordergroup_iter(binarytree_node)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -490,7 +493,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["3"], ["4", "6", "7"], []]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(
+            for group in iterators.levelordergroup_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
@@ -502,7 +505,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["2", "3"], ["4", "5", "6", "7"]]
         actual = [
             [node.node_name for node in group]
-            for group in levelordergroup_iter(
+            for group in iterators.levelordergroup_iter(
                 binarytree_node,
                 max_depth=3,
             )
@@ -512,7 +515,7 @@ class TestBinaryTreeIterator:
     @staticmethod
     def test_inorder_iter(binarytree_node):
         expected = ["4", "8", "2", "5", "1", "6", "3", "7"]
-        actual = [node.node_name for node in inorder_iter(binarytree_node)]
+        actual = [node.node_name for node in iterators.inorder_iter(binarytree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -520,7 +523,7 @@ class TestBinaryTreeIterator:
         expected = ["4", "1", "6", "3", "7"]
         actual = [
             node.node_name
-            for node in inorder_iter(
+            for node in iterators.inorder_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
@@ -530,13 +533,16 @@ class TestBinaryTreeIterator:
     @staticmethod
     def test_inorder_iter_max_depth(binarytree_node):
         expected = ["4", "2", "5", "1", "6", "3", "7"]
-        actual = [node.node_name for node in inorder_iter(binarytree_node, max_depth=3)]
+        actual = [
+            node.node_name
+            for node in iterators.inorder_iter(binarytree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
     def test_zigzag_iter(binarytree_node):
         expected = ["1", "3", "2", "4", "5", "6", "7", "8"]
-        actual = [node.node_name for node in zigzag_iter(binarytree_node)]
+        actual = [node.node_name for node in iterators.zigzag_iter(binarytree_node)]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -544,7 +550,7 @@ class TestBinaryTreeIterator:
         expected = ["1", "3", "4", "6", "7"]
         actual = [
             node.node_name
-            for node in zigzag_iter(
+            for node in iterators.zigzag_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
@@ -554,7 +560,10 @@ class TestBinaryTreeIterator:
     @staticmethod
     def test_zigzag_iter_max_depth(binarytree_node):
         expected = ["1", "3", "2", "4", "5", "6", "7"]
-        actual = [node.node_name for node in zigzag_iter(binarytree_node, max_depth=3)]
+        actual = [
+            node.node_name
+            for node in iterators.zigzag_iter(binarytree_node, max_depth=3)
+        ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
     @staticmethod
@@ -562,7 +571,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["3", "2"], ["4", "5", "6", "7"], ["8"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(binarytree_node)
+            for group in iterators.zigzaggroup_iter(binarytree_node)
         ]
         assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
 
@@ -571,7 +580,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["3"], ["4", "6", "7"], []]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(
+            for group in iterators.zigzaggroup_iter(
                 binarytree_node,
                 filter_condition=lambda x: x.node_name in ["1", "4", "3", "6", "7"],
             )
@@ -583,7 +592,7 @@ class TestBinaryTreeIterator:
         expected = [["1"], ["3", "2"], ["4", "5", "6", "7"]]
         actual = [
             [node.node_name for node in group]
-            for group in zigzaggroup_iter(
+            for group in iterators.zigzaggroup_iter(
                 binarytree_node,
                 max_depth=3,
             )
