@@ -1,11 +1,45 @@
 import unittest
+from typing import Any
 
+import matplotlib.pyplot as plt
 import pytest
 
-from bigtree.node.node import Node
-from bigtree.tree.construct import list_to_tree
-from bigtree.utils.iterators import postorder_iter
-from bigtree.utils.plot import first_pass, reingold_tilford
+from bigtree.node import node
+from bigtree.tree import construct
+from bigtree.utils import iterators, plot
+from tests.test_constants import Constants
+
+LOCAL = Constants.LOCAL
+
+
+class TestPlotTree(unittest.TestCase):
+    def test_plot_tree_runtime_error(self):
+        root = node.Node("a", children=[node.Node("b")])
+        with pytest.raises(RuntimeError) as exc_info:
+            plot.plot_tree(root)
+        assert str(exc_info.value) == Constants.ERROR_PLOT
+
+    def test_plot_tree_with_fig(self):
+        root = node.Node("a", children=[node.Node("b"), node.Node("c")])
+        plot.reingold_tilford(root)
+
+        fig = plt.figure()
+        ax = fig.add_subplot(111)
+        fig = plot.plot_tree(root, ax=ax)
+        if LOCAL:
+            fig.savefig(f"{Constants.LOCAL_FILE}/test_plot_tree_with_fig.png")
+        assert isinstance(fig, plt.Figure)
+
+    def test_plot_tree_with_fig_and_args(self):
+        root = node.Node("a", children=[node.Node("b"), node.Node("c")])
+        plot.reingold_tilford(root)
+
+        fig = plt.figure()
+        ax = fig.add_subplot(111)
+        fig = plot.plot_tree(root, "-ok", ax=ax)
+        if LOCAL:
+            fig.savefig(f"{Constants.LOCAL_FILE}/test_plot_tree_with_fig_and_args.png")
+        assert isinstance(fig, plt.Figure)
 
 
 class TestPlotNoChildren(unittest.TestCase):
@@ -16,7 +50,7 @@ class TestPlotNoChildren(unittest.TestCase):
         Tree should have structure
         a
         """
-        self.root = Node("a")
+        self.root = node.Node("a")
 
     def tearDown(self):
         self.root = None
@@ -25,17 +59,8 @@ class TestPlotNoChildren(unittest.TestCase):
         expected = [
             ("a", 0, 0, 0),
         ]
-        reingold_tilford(self.root)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("mod"),
-                node.get_attr("shift"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        plot.reingold_tilford(self.root)
+        assert_x_mod_shift(self.root, expected)
 
 
 class TestPlotShiftLeftSibling(unittest.TestCase):
@@ -44,21 +69,21 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
         This tests the shifting of the third subtree, which should trigger the shifting of left sibling
 
         Tree should have structure
-        o
-        ├── e
-        │   ├── a
-        │   └── d
-        │       ├── b
-        │       └── c
-        ├── f
-        └── n
-            ├── g
-            └── m
-                ├── h
-                ├── i
-                ├── j
-                ├── k
-                └── l
+                             ┌───┐
+                             │ o │
+                             └─┬─┘
+               ┌───────────────┼────────────────┐
+             ┌─┴─┐           ┌─┴─┐            ┌─┴─┐
+             │ e │           │ f │            │ n │
+             └─┬─┘           └───┘            └─┬─┘
+          ┌────┴────┐                 ┌─────────┴──────────┐
+        ┌─┴─┐     ┌─┴─┐             ┌─┴─┐                ┌─┴─┐
+        │ a │     │ d │             │ g │                │ m │
+        └───┘     └─┬─┘             └───┘                └─┬─┘
+                 ┌──┴───┐                    ┌──────┬──────┼──────┬──────┐
+               ┌─┴─┐  ┌─┴─┐                ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐
+               │ b │  │ c │                │ h │  │ i │  │ j │  │ k │  │ l │
+               └───┘  └───┘                └───┘  └───┘  └───┘  └───┘  └───┘
         """
         path_list = [
             "o/e/a",
@@ -72,7 +97,7 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             "o/n/m/k",
             "o/n/m/l",
         ]
-        root = list_to_tree(path_list)
+        root = construct.list_to_tree(path_list)
         self.root = root
 
     def tearDown(self):
@@ -99,21 +124,12 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 15, 12, 9),
             ("o", 13.5, 0, 0),
         ]
-        first_pass(
+        plot._first_pass(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("mod"),
-                node.get_attr("shift"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_mod_shift(self.root, expected)
 
     def test_reingold_tilford(self):
         expected = [
@@ -133,16 +149,8 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 4, 2),
             ("o", 2.25, 3),
         ]
-        reingold_tilford(self.root)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        plot.reingold_tilford(self.root)
+        assert_x_y_coordinate(self.root, expected)
 
     def test_reingold_tilford_with_separation(self):
         sibling_separation = 6
@@ -165,20 +173,12 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 15 + 9, 2),
             ("o", 13.5, 3),
         ]
-        reingold_tilford(
+        plot.reingold_tilford(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_y_coordinate(self.root, expected)
 
     def test_reingold_tilford_with_separation_and_offset(self):
         sibling_separation = 6
@@ -203,22 +203,14 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 24 + x_offset, 2 + y_offset),
             ("o", 13.5 + x_offset, 3 + y_offset),
         ]
-        reingold_tilford(
+        plot.reingold_tilford(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
             x_offset=x_offset,
             y_offset=y_offset,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_y_coordinate(self.root, expected)
 
     def test_reingold_tilford_with_separation_and_level_separation(self):
         sibling_separation = 6
@@ -242,21 +234,13 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 24, 2 * level_separation),
             ("o", 13.5, 3 * level_separation),
         ]
-        reingold_tilford(
+        plot.reingold_tilford(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
             level_separation=level_separation,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_y_coordinate(self.root, expected)
 
     def test_reingold_tilford_with_separation_and_offset_and_level_separation(self):
         sibling_separation = 6
@@ -282,7 +266,7 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             ("n", 24 + x_offset, 2 * level_separation + y_offset),
             ("o", 13.5 + x_offset, 3 * level_separation + y_offset),
         ]
-        reingold_tilford(
+        plot.reingold_tilford(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
@@ -290,15 +274,7 @@ class TestPlotShiftLeftSibling(unittest.TestCase):
             x_offset=x_offset,
             y_offset=y_offset,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_y_coordinate(self.root, expected)
 
 
 class TestPlotShiftRightSibling(unittest.TestCase):
@@ -307,32 +283,21 @@ class TestPlotShiftRightSibling(unittest.TestCase):
         This tests the shifting of the second subtree, which should trigger the shifting of right siblings
 
         Tree should have structure
-        z
-        ├── h
-        │   ├── c
-        │   │   ├── a
-        │   │   └── b
-        │   └── g
-        │       ├── d
-        │       ├── e
-        │       └── f
-        ├── p
-        │   ├── i
-        │   ├── j
-        │   └── o
-        │       ├── k
-        │       ├── l
-        │       ├── m
-        │       └── n
-        ├── q
-        ├── x
-        │   ├── r
-        │   ├── s
-        │   ├── t
-        │   ├── v
-        │   │   └── u
-        │   └── w
-        └── y
+                                                                         ┌───┐
+                                                                         │ z │
+                                                                         └─┬─┘
+                      ┌──────────────────────────────────┬─────────────────┴───────────┬────────────────────┬────────────────────┐
+                    ┌─┴─┐                              ┌─┴─┐                         ┌─┴─┐                ┌─┴─┐                ┌─┴─┐
+                    │ h │                              │ p │                         │ q │                │ x │                │ y │
+                    └─┬─┘                              └─┬─┘                         └───┘                └─┬─┘                └───┘
+             ┌────────┴────────┐             ┌──────┬────┴───────────┐                        ┌──────┬──────┼──────┬──────┐
+           ┌─┴─┐             ┌─┴─┐         ┌─┴─┐  ┌─┴─┐            ┌─┴─┐                    ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐
+           │ c │             │ g │         │ i │  │ j │            │ o │                    │ r │  │ s │  │ t │  │ v │  │ w │
+           └─┬─┘             └─┬─┘         └───┘  └───┘            └─┬─┘                    └───┘  └───┘  └───┘  └─┬─┘  └───┘
+          ┌──┴───┐      ┌──────┼──────┐                    ┌──────┬──┴───┬──────┐                                  │
+        ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐                ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐                              ┌─┴─┐
+        │ a │  │ b │  │ d │  │ e │  │ f │                │ k │  │ l │  │ m │  │ n │                              │ u │
+        └───┘  └───┘  └───┘  └───┘  └───┘                └───┘  └───┘  └───┘  └───┘                              └───┘
         """
         path_list = [
             "z/h/c/a",
@@ -354,7 +319,7 @@ class TestPlotShiftRightSibling(unittest.TestCase):
             "z/x/w",
             "z/y",
         ]
-        root = list_to_tree(path_list)
+        root = construct.list_to_tree(path_list)
         self.root = root
 
     def tearDown(self):
@@ -391,21 +356,12 @@ class TestPlotShiftRightSibling(unittest.TestCase):
             ("y", 5 + 1.75, 0, 4 * 5.75),
             ("z", 16.25, 0, 0),
         ]
-        first_pass(
+        plot._first_pass(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("mod"),
-                node.get_attr("shift"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_mod_shift(self.root, expected)
 
     def test_reingold_tilford_with_separation_and_offset_and_level_separation(self):
         sibling_separation = 1
@@ -442,7 +398,7 @@ class TestPlotShiftRightSibling(unittest.TestCase):
             ("y", 29.75 + x_offset, 2 * level_separation + y_offset),
             ("z", 16.25 + x_offset, 3 * level_separation + y_offset),
         ]
-        reingold_tilford(
+        plot.reingold_tilford(
             self.root,
             sibling_separation=sibling_separation,
             subtree_separation=subtree_separation,
@@ -450,15 +406,7 @@ class TestPlotShiftRightSibling(unittest.TestCase):
             x_offset=x_offset,
             y_offset=y_offset,
         )
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+        assert_x_y_coordinate(self.root, expected)
 
 
 class TestPlotRelativeShift(unittest.TestCase):
@@ -468,32 +416,21 @@ class TestPlotRelativeShift(unittest.TestCase):
         This also tests for nodes that will shift multiple times
 
         Tree should have structure
-        z
-        ├── g
-        │   ├── c
-        │   │   ├── a
-        │   │   └── b
-        │   └── f
-        │       ├── d
-        │       └── e
-        ├── h
-        ├── q
-        │   ├── i
-        │   ├── j
-        │   └── p
-        │       ├── k
-        │       ├── l
-        │       ├── m
-        │       ├── n
-        │       └── o
-        ├── x
-        │   ├── r
-        │   ├── s
-        │   ├── t
-        │   ├── v
-        │   │   └── u
-        │   └── w
-        └── y
+                                                                        ┌───┐
+                                                                        │ z │
+                                                                        └─┬─┘
+                    ┌─────────────────┬────────────────────┬──────────────┴─────────────────────────────────┬────────────────────┐
+                  ┌─┴─┐             ┌─┴─┐                ┌─┴─┐                                            ┌─┴─┐                ┌─┴─┐
+                  │ g │             │ h │                │ q │                                            │ x │                │ y │
+                  └─┬─┘             └───┘                └─┬─┘                                            └─┬─┘                └───┘
+             ┌──────┴──────┐                 ┌──────┬──────┴─────────────┐                    ┌──────┬──────┼──────┬──────┐
+           ┌─┴─┐         ┌─┴─┐             ┌─┴─┐  ┌─┴─┐                ┌─┴─┐                ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐
+           │ c │         │ f │             │ i │  │ j │                │ p │                │ r │  │ s │  │ t │  │ v │  │ w │
+           └─┬─┘         └─┬─┘             └───┘  └───┘                └─┬─┘                └───┘  └───┘  └───┘  └─┬─┘  └───┘
+          ┌──┴───┐      ┌──┴───┐                           ┌──────┬──────┼──────┬──────┐                           │
+        ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐                       ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐                       ┌─┴─┐
+        │ a │  │ b │  │ d │  │ e │                       │ k │  │ l │  │ m │  │ n │  │ o │                       │ u │
+        └───┘  └───┘  └───┘  └───┘                       └───┘  └───┘  └───┘  └───┘  └───┘                       └───┘
         """
         path_list = [
             "z/g/c/a",
@@ -515,7 +452,7 @@ class TestPlotRelativeShift(unittest.TestCase):
             "z/x/w",
             "z/y",
         ]
-        root = list_to_tree(path_list)
+        root = construct.list_to_tree(path_list)
         self.root = root
 
     def tearDown(self):
@@ -550,20 +487,8 @@ class TestPlotRelativeShift(unittest.TestCase):
             ("y", 5.5, 0, (1.5 + 1.5) + (4.5 + 4.5)),  # 9 shift
             ("z", 9.5, 0, 0),  # 8 x
         ]
-        first_pass(self.root, sibling_separation=1, subtree_separation=1)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("mod"),
-                node.get_attr("shift"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        for _actual, _expected in zip(actual, expected):
-            assert _actual == pytest.approx(
-                _expected, abs=0.1
-            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+        plot._first_pass(self.root, sibling_separation=1, subtree_separation=1)
+        assert_x_mod_shift(self.root, expected, approx=True)
 
     def test_reingold_tilford(self):
         expected = [
@@ -594,19 +519,8 @@ class TestPlotRelativeShift(unittest.TestCase):
             ("y", 17.5, 2),
             ("z", 9.5, 3),
         ]
-        reingold_tilford(self.root)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        for _actual, _expected in zip(actual, expected):
-            assert _actual == pytest.approx(
-                _expected, abs=0.1
-            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+        plot.reingold_tilford(self.root)
+        assert_x_y_coordinate(self.root, expected, approx=True)
 
 
 class TestPlotShiftLeftRightSibling(unittest.TestCase):
@@ -615,24 +529,21 @@ class TestPlotShiftLeftRightSibling(unittest.TestCase):
         This tests the shifting of the third subtree which should trigger shifting of second and fourth subtree.
 
         Tree should have structure
-        r
-        ├── g
-        │   ├── a
-        │   ├── b
-        │   ├── e
-        │   │   ├── c
-        │   │   └── d
-        │   └── f
-        ├── h
-        ├── n
-        │   ├── k
-        │   │   ├── i
-        │   │   └── j
-        │   ├── l
-        │   └── m
-        └── q
-            ├── o
-            └── p
+                                                   ┌───┐
+                                                   │ r │
+                                                   └─┬─┘
+                        ┌────────────────────┬───────┴──────────┬──────────────────┐
+                      ┌─┴─┐                ┌─┴─┐              ┌─┴─┐              ┌─┴─┐
+                      │ g │                │ h │              │ n │              │ q │
+                      └─┬─┘                └───┘              └─┬─┘              └─┬─┘
+          ┌──────┬──────┴──┬──────────┐                ┌────────┴─┬──────┐      ┌──┴───┐
+        ┌─┴─┐  ┌─┴─┐     ┌─┴─┐      ┌─┴─┐            ┌─┴─┐      ┌─┴─┐  ┌─┴─┐  ┌─┴─┐  ┌─┴─┐
+        │ a │  │ b │     │ e │      │ f │            │ k │      │ l │  │ m │  │ o │  │ p │
+        └───┘  └───┘     └─┬─┘      └───┘            └─┬─┘      └───┘  └───┘  └───┘  └───┘
+                        ┌──┴───┐                    ┌──┴───┐
+                      ┌─┴─┐  ┌─┴─┐                ┌─┴─┐  ┌─┴─┐
+                      │ c │  │ d │                │ i │  │ j │
+                      └───┘  └───┘                └───┘  └───┘
         """
         path_list = [
             "r/g/a",
@@ -648,7 +559,7 @@ class TestPlotShiftLeftRightSibling(unittest.TestCase):
             "r/q/o",
             "r/q/p",
         ]
-        root = list_to_tree(path_list)
+        root = construct.list_to_tree(path_list)
         self.root = root
 
     def tearDown(self):
@@ -675,20 +586,8 @@ class TestPlotShiftLeftRightSibling(unittest.TestCase):
             ("q", 4.5, 4, (1.5 + 1.5 / 2) + 2.25),
             ("r", 5.25, 0, 0),
         ]
-        first_pass(self.root, sibling_separation=1, subtree_separation=1)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("mod"),
-                node.get_attr("shift"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        for _actual, _expected in zip(actual, expected):
-            assert _actual == pytest.approx(
-                _expected, abs=0.1
-            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+        plot._first_pass(self.root, sibling_separation=1, subtree_separation=1)
+        assert_x_mod_shift(self.root, expected, approx=True)
 
     def test_reingold_tilford(self):
         expected = [
@@ -711,19 +610,8 @@ class TestPlotShiftLeftRightSibling(unittest.TestCase):
             ("q", 9, 2),
             ("r", 5.25, 3),
         ]
-        reingold_tilford(self.root)
-        actual = [
-            (
-                node.node_name,
-                node.get_attr("x"),
-                node.get_attr("y"),
-            )
-            for node in postorder_iter(self.root)
-        ]
-        for _actual, _expected in zip(actual, expected):
-            assert _actual == pytest.approx(
-                _expected, abs=0.1
-            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+        plot.reingold_tilford(self.root)
+        assert_x_y_coordinate(self.root, expected, approx=True)
 
 
 class TestPlotNonNegative(unittest.TestCase):
@@ -732,19 +620,21 @@ class TestPlotNonNegative(unittest.TestCase):
         This tests the leftmost node having negative x-coordinates and the third pass will have to adjust the tree accordingly.
 
         Tree should have structure
-        m
-        ├── a
-        ├── b
-        ├── k
-        │   ├── c
-        │   ├── d
-        │   ├── g
-        │   │   ├── e
-        │   │   └── f
-        │   ├── h
-        │   ├── i
-        │   └── j
-        └── l
+                                       ┌───┐
+                                       │ m │
+                                       └─┬─┘
+          ┌──────┬───────────────────────┴───┬───────────────────────────┐
+        ┌─┴─┐  ┌─┴─┐                       ┌─┴─┐                       ┌─┴─┐
+        │ a │  │ b │                       │ k │                       │ l │
+        └───┘  └───┘                       └─┬─┘                       └───┘
+                        ┌──────┬─────────┬───┴──────┬──────┬──────┐
+                      ┌─┴─┐  ┌─┴─┐     ┌─┴─┐      ┌─┴─┐  ┌─┴─┐  ┌─┴─┐
+                      │ c │  │ d │     │ g │      │ h │  │ i │  │ j │
+                      └───┘  └───┘     └─┬─┘      └───┘  └───┘  └───┘
+                                      ┌──┴───┐
+                                    ┌─┴─┐  ┌─┴─┐
+                                    │ e │  │ f │
+                                    └───┘  └───┘
         """
         path_list = [
             "m/a",
@@ -758,7 +648,7 @@ class TestPlotNonNegative(unittest.TestCase):
             "m/k/j",
             "m/l",
         ]
-        root = list_to_tree(path_list)
+        root = construct.list_to_tree(path_list)
         self.root = root
 
     def tearDown(self):
@@ -780,7 +670,7 @@ class TestPlotNonNegative(unittest.TestCase):
             ("l", 3, 0, 0),
             ("m", 1.5, 0, 0),
         ]
-        first_pass(self.root, sibling_separation=1, subtree_separation=1)
+        plot._first_pass(self.root, sibling_separation=1, subtree_separation=1)
         actual = [
             (
                 node.node_name,
@@ -788,7 +678,7 @@ class TestPlotNonNegative(unittest.TestCase):
                 node.get_attr("mod"),
                 node.get_attr("shift"),
             )
-            for node in postorder_iter(self.root)
+            for node in iterators.postorder_iter(self.root)
         ]
         for _actual, _expected in zip(actual, expected):
             assert _actual == pytest.approx(
@@ -811,16 +701,193 @@ class TestPlotNonNegative(unittest.TestCase):
             ("l", 3 + 0.5, 2),
             ("m", 1.5 + 0.5, 3),
         ]
-        reingold_tilford(self.root)
+        plot.reingold_tilford(self.root)
+        assert_x_y_coordinate(self.root, expected, approx=True)
+
+
+class TestPlotTreeCumulativeShift(unittest.TestCase):
+    def setUp(self):
+        """
+        This tests when the left sub_tree need to take into account the cumulative would-be shift to calculate new_shift.
+
+        Tree should have structure
+                                                                                                          ┌───┐
+                                                                                                          │ a │
+                                                                                                          └─┬─┘
+                          ┌────────────────────────────┬──────────────────────────────────────────────┬─────┴──────────────────────────────────────────────────────────────────────────────────┐
+                        ┌─┴─┐                        ┌─┴─┐                                          ┌─┴─┐                                                                                    ┌─┴─┐
+                        │ b │                        │ c │                                          │ d │                                                                                    │ e │
+                        └─┬─┘                        └─┬─┘                                          └─┬─┘                                                                                    └─┬─┘
+          ┌───────┬───────┼───────┬───────┐            │            ┌───────┬───────┬───────┬─────────┴───────────┬─────────────────────┐                              ┌───────────────────────┴──────┬─────────────────┐
+        ┌─┴──┐  ┌─┴──┐  ┌─┴──┐  ┌─┴──┐  ┌─┴──┐       ┌─┴──┐       ┌─┴──┐  ┌─┴──┐  ┌─┴──┐  ┌─┴──┐                ┌─┴──┐                ┌─┴──┐                         ┌─┴──┐                         ┌─┴──┐            ┌─┴──┐
+        │ b1 │  │ b2 │  │ b3 │  │ b4 │  │ b5 │       │ c1 │       │ d1 │  │ d2 │  │ d3 │  │ d4 │                │ d5 │                │ d6 │                         │ e1 │                         │ e2 │            │ e3 │
+        └────┘  └────┘  └────┘  └────┘  └────┘       └─┬──┘       └────┘  └────┘  └────┘  └────┘                └─┬──┘                └────┘                         └─┬──┘                         └────┘            └─┬──┘
+                                                   ┌───┴────┐                                        ┌────────┬───┴────┬────────┐                ┌────────┬────────┬───┴────┬────────┬────────┐                ┌────────┼────────┐
+                                                ┌──┴──┐  ┌──┴──┐                                  ┌──┴──┐  ┌──┴──┐  ┌──┴──┐  ┌──┴──┐          ┌──┴──┐  ┌──┴──┐  ┌──┴──┐  ┌──┴──┐  ┌──┴──┐  ┌──┴──┐          ┌──┴──┐  ┌──┴──┐  ┌──┴──┐
+                                                │ c11 │  │ c12 │                                  │ d51 │  │ d52 │  │ d53 │  │ d54 │          │ e11 │  │ e12 │  │ e13 │  │ e14 │  │ e15 │  │ e16 │          │ e31 │  │ e32 │  │ e33 │
+                                                └─────┘  └─────┘                                  └─────┘  └─────┘  └─────┘  └─────┘          └─────┘  └─────┘  └─────┘  └─────┘  └─────┘  └─────┘          └─────┘  └─────┘  └─────┘
+        """
+        path_list = [
+            "a/b/b1",
+            "a/b/b2",
+            "a/b/b3",
+            "a/b/b4",
+            "a/b/b5",
+            "a/c/c1/c11",
+            "a/c/c1/c12",
+            "a/d/d1",
+            "a/d/d2",
+            "a/d/d3",
+            "a/d/d4",
+            "a/d/d5/d51",
+            "a/d/d5/d52",
+            "a/d/d5/d53",
+            "a/d/d5/d54",  # left subtree
+            "a/d/d6",
+            "a/e/e1/e11",  # right subtree
+            "a/e/e1/e12",
+            "a/e/e1/e13",
+            "a/e/e1/e14",
+            "a/e/e1/e15",
+            "a/e/e1/e16",
+            "a/e/e2",
+            "a/e/e3/e31",
+            "a/e/e3/e32",
+            "a/e/e3/e33",
+        ]
+        root = construct.list_to_tree(path_list)
+        self.root = root
+
+    def tearDown(self):
+        self.root = None
+
+    def test_first_pass(self):
+        expected = [
+            ("b1", 0.0, 0.0, 0.0),
+            ("b2", 1.0, 0.0, 0.0),
+            ("b3", 2.0, 0.0, 0.0),
+            ("b4", 3.0, 0.0, 0.0),
+            ("b5", 4.0, 0.0, 0.0),
+            ("b", 2.0, 0.0, 0.0),
+            ("c11", 0.0, 0.0, 0.0),
+            ("c12", 1.0, 0.0, 0.0),
+            ("c1", 0.5, 0.0, 0.0),
+            ("c", 3.0, 2.5, 7.75),
+            ("d1", 0.0, 0.0, 0.0),
+            ("d2", 1.0, 0.0, 0.0),
+            ("d3", 2.0, 0.0, 0.0),
+            ("d4", 3.0, 0.0, 0.0),
+            ("d51", 0.0, 0.0, 0.0),
+            ("d52", 1.0, 0.0, 0.0),
+            ("d53", 2.0, 0.0, 0.0),
+            ("d54", 3.0, 0.0, 0.0),
+            ("d5", 4.0, 2.5, 0.0),
+            ("d6", 5.0, 0.0, 0.0),
+            ("d", 4.0, 1.5, 15.5),
+            ("e11", 0.0, 0.0, 0.0),
+            ("e12", 1.0, 0.0, 0.0),
+            ("e13", 2.0, 0.0, 0.0),
+            ("e14", 3.0, 0.0, 0.0),
+            ("e15", 4.0, 0.0, 0.0),
+            ("e16", 5.0, 0.0, 0.0),
+            ("e1", 2.5, 0.0, 0.0),
+            ("e2", 3.5, 0.0, 1.25),
+            ("e31", 0.0, 0.0, 0.0),
+            ("e32", 1.0, 0.0, 0.0),
+            ("e33", 2.0, 0.0, 0.0),
+            ("e3", 4.5, 3.5, 2.5),
+            ("e", 5.0, 0.25, 23.25),
+            ("a", 15.125, 0.0, 0.0),
+        ]
+        plot._first_pass(self.root, sibling_separation=1, subtree_separation=1)
         actual = [
             (
                 node.node_name,
                 node.get_attr("x"),
-                node.get_attr("y"),
+                node.get_attr("mod"),
+                node.get_attr("shift"),
             )
-            for node in postorder_iter(self.root)
+            for node in iterators.postorder_iter(self.root)
         ]
         for _actual, _expected in zip(actual, expected):
             assert _actual == pytest.approx(
                 _expected, abs=0.1
             ), f"Expected\n{_expected}\nReceived\n{_actual}"
+
+    def test_reingold_tilford(self):
+        expected = [
+            ("b1", 0.0, 1.0),
+            ("b2", 1.0, 1.0),
+            ("b3", 2.0, 1.0),
+            ("b4", 3.0, 1.0),
+            ("b5", 4.0, 1.0),
+            ("b", 2.0, 2.0),
+            ("c11", 10.25, 0.0),
+            ("c12", 11.25, 0.0),
+            ("c1", 10.75, 1.0),
+            ("c", 10.75, 2.0),
+            ("d1", 17.0, 1.0),
+            ("d2", 18.0, 1.0),
+            ("d3", 19.0, 1.0),
+            ("d4", 20.0, 1.0),
+            ("d51", 19.5, 0.0),
+            ("d52", 20.5, 0.0),
+            ("d53", 21.5, 0.0),
+            ("d54", 22.5, 0.0),
+            ("d5", 21.0, 1.0),
+            ("d6", 22.0, 1.0),
+            ("d", 19.5, 2.0),
+            ("e11", 23.5, 0.0),
+            ("e12", 24.5, 0.0),
+            ("e13", 25.5, 0.0),
+            ("e14", 26.5, 0.0),
+            ("e15", 27.5, 0.0),
+            ("e16", 28.5, 0.0),
+            ("e1", 26, 1.0),
+            ("e2", 28.25, 1.0),
+            ("e31", 29.5, 0.0),
+            ("e32", 30.5, 0.0),
+            ("e33", 31.5, 0.0),
+            ("e3", 30.5, 1.0),
+            ("e", 28.25, 2.0),
+            ("a", 15.125, 3.0),
+        ]
+        plot.reingold_tilford(self.root)
+        assert_x_y_coordinate(self.root, expected, approx=True)
+
+
+def assert_x_y_coordinate(tree: node.Node, expected: list[Any], approx: bool = False):
+    actual = [
+        (
+            _node.node_name,
+            _node.get_attr("x"),
+            _node.get_attr("y"),
+        )
+        for _node in iterators.postorder_iter(tree)
+    ]
+    if approx:
+        for _actual, _expected in zip(actual, expected):
+            assert _actual == pytest.approx(
+                _expected, abs=0.1
+            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+    else:
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
+
+
+def assert_x_mod_shift(tree: node.Node, expected: list[Any], approx: bool = False):
+    actual = [
+        (
+            _node.node_name,
+            _node.get_attr("x"),
+            _node.get_attr("mod"),
+            _node.get_attr("shift"),
+        )
+        for _node in iterators.postorder_iter(tree)
+    ]
+    if approx:
+        for _actual, _expected in zip(actual, expected):
+            assert _actual == pytest.approx(
+                _expected, abs=0.1
+            ), f"Expected\n{_expected}\nReceived\n{_actual}"
+    else:
+        assert actual == expected, f"Expected\n{expected}\nReceived\n{actual}"
